unit RRPGAdPanel;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, RRPGURLGetter,
  RRPGUrlPicture, StdCtrls, ExtCtrls, RRPGDrawUtil, RRPGColorScheme,
  RRPGControls, GIFImg, ShellApi; // DBCtrls

type
  TOnGetIDUnico = procedure (Sender: TObject; var IdUnico: String) of object;

  TThreadLoadURL = class(TThread)
  private
    FURL: String;
  protected
    procedure Execute; override;
  public
    constructor Create(ASuspenso: Boolean; URL: String);
  end;

  TRAdPanel = class(TRContentPanel)
  private
    FAdGetter: TRRPGURLGetter;
    FImgGetter: TRRPGURLGetter;
    FTimerRecoverAdErrorLoad: TTimer;
    FTimerAdGetter: TTimer;

    // Imagem
    FCompImg: TImage;

    // Texto
    FCompPan: TPanel;
    FCompTitulo: TRLabel;
    FCompLinha1: TRLabel;
    FCompLinha2: TRLabel;
    FCompURLPreview: TRLabel;

    FAtualID: Int64;

    FNovaID: Int64;

    FOnGetIDUnico: TOnGetIDUnico;
  protected
    procedure DoObterAdDados; virtual;
    procedure SetAdAtualBanner(NovaID: Int64; NovoURLImg: String); virtual;
    procedure SetAdAtualTexto(NovaID: Int64; Titulo, Linha1, Linha2, URLPreview: String); virtual;

    procedure OnDadosAdGetter(const Arquivo: String); virtual;
    procedure OnDadosImgGetter(const Arquivo: String); virtual;
    procedure OnErroImgGetter(Sender: TObject); virtual;
    procedure OnTimeAdGetter(Sender: TObject); virtual;
    procedure OnTimeRecoverError(Sender: TObject); virtual;
    procedure OnAnuncioClick(Sender: TObject); virtual;
    procedure CriarComponentesTexto; virtual;

    procedure ConfigurarVisual; override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure CarregarOutroAnuncio;
  published
    property OnGetIDUnico: TOnGetIDUnico read FOnGetIDUnico write FOnGetIDUnico;
  end;

var
  URL_ADCONTROLLER: String = 'http://firecast.rrpg.com.br:90/adController';

implementation

{$IF RTLVERSION >= 25}
uses
  UITypes;
{$IFEND}

{ TRAdPanel }

procedure TRAdPanel.CarregarOutroAnuncio;
begin
  Self.DoObterAdDados();
end;

procedure TRAdPanel.ConfigurarVisual;
begin
  inherited;
  Self.Alpha := 255;
end;

constructor TRAdPanel.Create(AOwner: TComponent);
begin
  inherited;

  if (csDesigning in Self.ComponentState) and
     (not (csReading in Self.ComponentState)) then
  begin
    Self.ClientWidth := 234;
    Self.ClientHeight := 60;
  end;

  FAdGetter := TRRPGURLGetter.Create;
  FAdGetter.OnDados := Self.OnDadosAdGetter;

  FImgGetter := TRRPGURLGetter.Create;
  FImgGetter.OnDados := Self.OnDadosImgGetter;
  FImgGetter.OnErro := Self.OnErroImgGetter;

  FTimerAdGetter := TTimer.Create(nil);
  FTimerAdGetter.Interval := 4 * 60 * 1000;
  FTimerAdGetter.OnTimer := Self.OnTimeAdGetter;

  if not (csDesigning in Self.ComponentState) then
  begin
    FTimerRecoverAdErrorLoad := TTimer.Create(nil);
    FTimerRecoverAdErrorLoad.Enabled := False;
    FTimerRecoverAdErrorLoad.Interval := 1000;
    FTimerRecoverAdErrorLoad.OnTimer := Self.OnTimeRecoverError;

    FCompImg := TImage.Create(nil);
    FCompImg.Visible := False;
    FCompImg.Cursor := crHandPoint;
    FCompImg.Align := alClient;
    FCompImg.Center := True;
    FCompImg.OnClick := Self.OnAnuncioClick;

    Self.CriarComponentesTexto();

    FTimerAdGetter.Enabled := True;
    Self.DoObterAdDados();
  end else
    FTimerAdGetter.Enabled := False;
end;

procedure TRAdPanel.CriarComponentesTexto;
begin
  FCompPan := TPanel.Create(nil);
  FCompPan.ParentColor := True;
  FCompPan.Caption := '';
  FCompPan.BevelInner := bvNone;
  FCompPan.BevelOuter := bvNone;
  FCompPan.Visible := False;
  FCompPan.Align := alClient;

  FCompTitulo := TRLabel.Create(nil);
  FCompTitulo.Align := alTop;
  FCompTitulo.Estilo := elLink;
  FCompTitulo.Cursor := crHandPoint;
  FCompTitulo.OnClick := Self.OnAnuncioClick;
  FCompTitulo.Font.Style := FCompTitulo.Font.Style + [fsUnderline];

  FCompLinha1 := TRLabel.Create(nil);
  FCompLinha1.Align := alTop;

  FCompLinha2 := TRLabel.Create(nil);
  FCompLinha2.Align := alTop;

  FCompURLPreview := TRLabel.Create(nil);
  FCompURLPreview.Align := alTop;
  FCompURLPreview.Estilo := elLinkDefinicao;
  FCompURLPreview.OnClick := Self.OnAnuncioClick;

  FCompPan.BorderWidth := 2;
  FCompPan.InsertControl(FCompTitulo);
  FCompPan.InsertControl(FCompLinha1);
  FCompPan.InsertControl(FCompLinha2);
  FCompPan.InsertControl(FCompURLPreview);
end;

destructor TRAdPanel.Destroy;
begin
  FreeAndNil(FCompTitulo);
  FreeAndNil(FCompLinha1);
  FreeAndNil(FCompLinha2);
  FreeAndNil(FCompURLPreview);
  FreeAndNil(FCompPan);

  if FTimerRecoverAdErrorLoad <> nil then
    FTimerRecoverAdErrorLoad.Free;

  if FCompImg <> nil then
    FCompImg.Free;

  FTimerAdGetter.Free;
  FImgGetter.Free;
  FAdGetter.Free;
  inherited;
end;

procedure TRAdPanel.DoObterAdDados;
var
  URLFinal, DataStr: String;
begin
  DataStr := FormatDateTime('ddmmyyhhnnsszzz', Now); // Evitar o uso de Cache
  URLFinal := URL_ADCONTROLLER + '?op=get&random=' + IntToStr(Random(9999999)) +
              '&datalocal=' + DataStr;

  FAdGetter.CarregarURL(URLFinal);
end;

procedure TRAdPanel.Loaded;
begin
  inherited;

  if Self.ClientWidth <> 234 then
    Self.ClientWidth := 234;

  if Self.ClientHeight <> 60 then
    Self.ClientHeight := 60;
end;

procedure TRAdPanel.OnDadosAdGetter(const Arquivo: String);
var
  StrL: TStringList;
  ID: Int64;
  URL, Tipo: String;
begin
  try
    StrL := TStringList.Create;

    try
      StrL.LoadFromFile(Arquivo);

      ID := StrToIntDef(StrL.Values['id'], 0);
      Tipo := UpperCase(StrL.Values['tipo']);

      if Tipo = 'B' then
      begin
        URL := StrL.Values['urlimg'];
        Self.SetAdAtualBanner(ID, URL);
      end else
        if Tipo = 'T' then
        begin
          Self.SetAdAtualTexto(ID, StrL.Values['titulo'], StrL.Values['linha1'],
                               StrL.Values['linha2'], StrL.Values['urlpreview']);
        end;
    finally
      StrL.Free;
    end;
  except
    // Ignorar exception
  end;
end;

procedure TRAdPanel.OnDadosImgGetter(const Arquivo: String);
begin
  try
    if FCompPan.Visible then
      FCompPan.Visible := False;

    FCompImg.Picture.LoadFromFile(Arquivo);

    if FCompImg.Parent <> Self then
      FCompImg.Parent := Self;

    FCompImg.Visible := True;

    if FCompImg.Picture.Graphic <> nil then
      if FCompImg.Picture.Graphic is TGIFImage then
        TGIFImage(FCompImg.Picture.Graphic).Animate := True;
  except
    FTimerRecoverAdErrorLoad.Enabled := True;
    Exit;
  end;

  FAtualID := FNovaID;
end;

procedure TRAdPanel.OnErroImgGetter(Sender: TObject);
begin
  FTimerRecoverAdErrorLoad.Enabled := True;
end;

procedure TRAdPanel.OnAnuncioClick(Sender: TObject);
var
  URL, ID, IDNovo: String;
  i: Integer;
begin
  URL := URL_ADCONTROLLER + '?op=goto&id=' + IntToStr(FAtualID);

  ID := '';

  if Assigned(FOnGetIDUnico) then
    FOnGetIDUnico(Self, ID);

  IDNovo := '';

  for i := 1 to Length(ID) do
    if CharInSet(ID[i], ['0'..'9', 'a'..'z', 'A'..'Z']) then
      IDNovo := IDNovo + ID[i]
    else
      IDNovo := IDNovo + Format('%%%.2x', [Ord(ID[i])]);

  if IDNovo <> '' then
    URL := URL + '&idunico=' + IDNovo;

  TThreadLoadURL.Create(False, URL);
end;

procedure TRAdPanel.OnTimeAdGetter(Sender: TObject);
begin
  Self.DoObterAdDados();
end;

procedure TRAdPanel.OnTimeRecoverError(Sender: TObject);
begin
  FTimerRecoverAdErrorLoad.Enabled := False;
  Self.DoObterAdDados();
end;

procedure TRAdPanel.SetAdAtualBanner(NovaID: Int64; NovoURLImg: String);
begin
  FNovaID := NovaID;
  FImgGetter.CarregarURL(NovoURLImg);
end;

procedure TRAdPanel.SetAdAtualTexto(NovaID: Int64; Titulo, Linha1,
  Linha2, URLPreview: String);
begin
  FAtualID := NovaID;

  if FCompImg.Visible then
    FCompImg.Visible := False;

  if FCompPan.Parent <> Self then
    FCompPan.Parent := Self;

  FCompTitulo.Caption := Titulo;
  FCompLinha1.Caption := Linha1;
  FCompLinha2.Caption := Linha2;
  FCompURLPreview.Caption := URLPreview;

  FCompPan.Visible := True;
end;

{ TThreadLoadURL }

constructor TThreadLoadURL.Create(ASuspenso: Boolean; URL: String);
begin
  FURL := URL;
  Self.FreeOnTerminate := True;
  inherited Create(ASuspenso);
end;

procedure TThreadLoadURL.Execute;
begin
  try
    Self.FreeOnTerminate := True;
    ShellApi.ShellExecute(0, 'open', PWideChar(FURL), nil, nil, SW_NORMAL);
  except
    on E: Exception do
    begin
      //Ignorar
    end;
  end;
end;

end.
