unit RRPGBaseModuleCreator;

interface

uses
  DesignIntf, ToolsApi, Classes, SysUtils, DesignEditors;

type
  TRBaseModuleCreator = class;


  TRBaseModuleWizard = class(TInterfacedObject, IOTAWizard)
  public
    procedure AfterSave; virtual;
    { This function is called immediately before the item is saved. This is not
      called for IOTAWizard }
    procedure BeforeSave; virtual;
    { The associated item is being destroyed so all references should be dropped.
      Exceptions are ignored. }
    procedure Destroyed; virtual;
    { This associated item was modified in some way. This is not called for
      IOTAWizards }
    procedure Modified; virtual;

    function GetIDString: string; virtual;
    function GetName: string; virtual;
    function GetState: TWizardState; virtual;

    { Launch the AddIn }

    procedure Execute; virtual;
    procedure OnClick(Sender: TObject); virtual;

    function GetAuthor: string; virtual;
    function GetComment: string; virtual;
    function GetPage: string; virtual;
    function GetGlyph: Cardinal; virtual;


    procedure ExecutarCreator(Creator: IOTACreator); virtual;
  end;

  TRBaseModuleCreator = class(TInterfacedObject, IOTACreator, IOTAModuleCreator,
    IOTAAdditionalFilesModuleCreator)
  public
    function GetCreatorType: string; virtual;
    { Return False if this is a new module }
    function GetExisting: Boolean; virtual;
    { Return the File system IDString that this module uses for reading/writing }
    function GetFileSystem: string; virtual;
    { Return the Owning module, if one exists (for a project module, this would
      be a project; for a project this is a project group) }
    function GetOwner: IOTAModule; virtual;
    { Return true, if this item is to be marked as un-named.  This will force the
      save as dialog to appear the first time the user saves. }
    function GetUnnamed: Boolean; virtual;

    function GetAncestorName: string; virtual;
    { Return the implementation filename, or blank to have the IDE create a new
      unique one. (C++ .cpp file or Delphi unit) NOTE: If a value is returned then it *must* be a
      fully qualified filename.  This also applies to GetIntfFileName and
      GetAdditionalFileName on the IOTAAdditionalFilesModuleCreator interface. }
    function GetImplFileName: string; virtual;
    { Return the interface filename, or blank to have the IDE create a new
      unique one.  (C++ header) }
    function GetIntfFileName: string; virtual;
    { Return the form name }
    function GetFormName: string; virtual;
    { Return True to Make this module the main form of the given Owner/Project }
    function GetMainForm: Boolean; virtual;
    { Return True to show the form }
    function GetShowForm: Boolean; virtual;
    { Return True to show the source }
    function GetShowSource: Boolean; virtual;
    { Create and return the Form resource for this new module if applicable }
    function NewFormFile(const FormIdent, AncestorIdent: string): IOTAFile; virtual;
    { Create and return the Implementation source for this module. (C++ .cpp
      file or Delphi unit) }
    function NewImplSource(const ModuleIdent, FormIdent, AncestorIdent: string): IOTAFile; virtual;
    { Create and return the Interface (C++ header) source for this module }
    function NewIntfSource(const ModuleIdent, FormIdent, AncestorIdent: string): IOTAFile; virtual;
    { Called when the new form/datamodule/custom module is created }
    procedure FormCreated(const FormEditor: IOTAFormEditor); virtual;

    function GetAdditionalFilesCount: Integer; virtual;
    { Create and return the source for this additional file }
    function NewAdditionalFileSource(I: Integer; const ModuleIdent, FormIdent, AncestorIdent: string): IOTAFile; virtual;
    { Return the additional filename, or blank to have the IDE create a new
      unique one. }
    function GetAdditionalFileName(I: Integer): string; virtual;
    { Get the file extent used to create a new file name }
    function GetAdditionalFileExt(I: Integer): string; virtual;
  end;

implementation


{ TMIdHTTPServWizard }

procedure TRBaseModuleWizard.AfterSave;
begin

end;

procedure TRBaseModuleWizard.BeforeSave;
begin

end;

procedure TRBaseModuleWizard.Destroyed;
begin

end;

procedure TRBaseModuleWizard.ExecutarCreator(Creator: IOTACreator);
var
  MServices: IOTAModuleServices;
begin
  MServices := ToolsApi.BorlandIDEServices as IOTAModuleServices;

  if MServices = nil then
    Exit;

  MServices.CreateModule(Creator);
end;

procedure TRBaseModuleWizard.Execute;
begin
end;

function TRBaseModuleWizard.GetAuthor: string;
begin
  Result := 'RRPG'
end;

function TRBaseModuleWizard.GetComment: string;
begin
  Result := '';
end;

function TRBaseModuleWizard.GetGlyph: Cardinal;
begin
  Result := 0;
end;

function TRBaseModuleWizard.GetIDString: string;
begin
  Result := Self.ClassName;
end;

function TRBaseModuleWizard.GetName: string;
begin
  Result := '';
end;

function TRBaseModuleWizard.GetPage: string;
begin
  Result := 'RRPG';
end;

function TRBaseModuleWizard.GetState: TWizardState;
begin
  Result := [];
end;

procedure TRBaseModuleWizard.Modified;
begin

end;

procedure TRBaseModuleWizard.OnClick(Sender: TObject);
begin
  Self.Execute;
end;

{ TMIdHTTPServCreator }

procedure TRBaseModuleCreator.FormCreated(const FormEditor: IOTAFormEditor);
begin

end;

function TRBaseModuleCreator.GetAdditionalFileExt(I: Integer): string;
begin
  Result := '';
end;

function TRBaseModuleCreator.GetAdditionalFileName(I: Integer): string;
begin
  Result := ''
end;

function TRBaseModuleCreator.GetAdditionalFilesCount: Integer;
begin
  Result := 0;
end;

function TRBaseModuleCreator.GetAncestorName: string;
begin
  Result := '';
end;

function TRBaseModuleCreator.GetCreatorType: string;
begin
  Result := sForm;
end;

function TRBaseModuleCreator.GetExisting: Boolean;
begin
  Result := False;
end;

function TRBaseModuleCreator.GetFileSystem: string;
begin
  Result := '';
end;

function TRBaseModuleCreator.GetFormName: string;
begin
  Result := '';
end;

function TRBaseModuleCreator.GetImplFileName: string;
begin
  Result := '';
end;

function TRBaseModuleCreator.GetIntfFileName: string;
begin
  Result := '';
end;

function TRBaseModuleCreator.GetMainForm: Boolean;
begin
  Result := False;
end;

function TRBaseModuleCreator.GetOwner: IOTAModule;
begin
  Result := GetActiveProject;
end;

function TRBaseModuleCreator.GetShowForm: Boolean;
begin
  Result := True;
end;

function TRBaseModuleCreator.GetShowSource: Boolean;
begin
  Result := True;
end;

function TRBaseModuleCreator.GetUnnamed: Boolean;
begin
  Result := True;
end;

function TRBaseModuleCreator.NewAdditionalFileSource(I: Integer;
  const ModuleIdent, FormIdent, AncestorIdent: string): IOTAFile;
begin
  Result := nil;
end;

function TRBaseModuleCreator.NewFormFile(const FormIdent,
  AncestorIdent: string): IOTAFile;
var
  Conteudo: String;
begin
  Conteudo := Format(
    'inherited %0:s: T%0:s'#13#10+
    '  OldCreateOrder = False'#13#10+
    '  Left = 188'#13#10+
    '  Top = 114'#13#10+
    '  Height = 150'#13#10+
    '  Width = 215'#13#10+
    'end', [FormIdent]);

  Result := StringToIOTAFile(Conteudo);
end;

function TRBaseModuleCreator.NewImplSource(const ModuleIdent, FormIdent,
  AncestorIdent: string): IOTAFile;
begin
  Result := nil;
end;

function TRBaseModuleCreator.NewIntfSource(const ModuleIdent, FormIdent,
  AncestorIdent: string): IOTAFile;
begin
  Result := nil;
end;

end.
