unit RRPGDataSet;

interface

uses
  RRPGBaseDataSet, RRPGDataSetUtil, DB, Classes, SysUtils;

type
  {$if compilerversion < 23} // Menor que Delphi XE2
    NativeInt = Integer;
  {$ifend}

  ERRPGSomenteLeitura = class(EAbort);

  TRRPGDataSet = class(TRCustomDataSet)
  private
    // Controles
    FAberto: Boolean;
    FSomenteLeitura: Boolean;
    procedure SetSomenteLeitura(const Value: Boolean);
  protected
    procedure NotificarAlteracaoDataSet; virtual;
    procedure InternalOpen; override;
    procedure InternalClose; override;
    function GetRecordCount: Integer; override;
    function IsCursorOpen: Boolean; override;

    function InternoAlimentarProximoRecordBuffer(var Buf: TBufferRecord): Boolean; override;
    function InternoGetFieldCountForBuffer: Integer; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalPost; override;
    procedure InternalDelete; override;
    procedure DoBeforeInsert; override;
    procedure DoBeforeEdit; override;
    procedure DoBeforeDelete; override;
  protected
    procedure DoAfterEdit; override;
    procedure DoAfterCancel; override;
    procedure DoAfterInsert; override;
    procedure DoAfterDelete; override;
    function GetCanModify: Boolean; override;
    procedure DataEvent(Event: TDataEvent; Info: NativeInt); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure AbrirNormal; virtual;
    procedure LimparDataSet; virtual;
  published
    property BeforeScroll;
    property BeforeRefresh;
    property BeforePost;
    property BeforeOpen;
    property BeforeInsert;
    property BeforeEdit;
    property BeforeDelete;
    property BeforeClose;
    property BeforeCancel;
    property AfterScroll;
    property AfterRefresh;
    property AfterPost;
    property AfterOpen;
    property AfterInsert;
    property AfterEdit;
    property AfterDelete;
    property AfterClose;
    property AfterCancel;
    property OnPostError;
    property OnNewRecord;
    property OnEditError;
    property OnDeleteError;
    property OnCalcFields;
    property Active;

    property SomenteLeitura: Boolean read FSomenteLeitura write SetSomenteLeitura;
  end;

implementation

uses
  Dialogs, RRPGOwnerDatasets;

constructor TRRPGDataSet.Create(AOwner: TComponent);
begin
  inherited;

end;

procedure TRRPGDataSet.DataEvent(Event: TDataEvent; Info: NativeInt);
var
  ChamarInherited: Boolean;
begin
  ChamarInherited := True;

  case Event of
    deUpdateState:
    begin
      if Self.ControlsDisabled then
        ChamarInherited := False;
    end;
  end;


  if ChamarInherited then
    inherited;
end;

destructor TRRPGDataSet.Destroy;
begin

  inherited;
end;

function TRRPGDataSet.GetCanModify: Boolean;
begin
  Result := (inherited GetCanModify) and (not Self.FSomenteLeitura);
end;

function TRRPGDataSet.GetRecordCount: Integer;
begin
  Result := Self.TamanhoBuffer;
end;

procedure TRRPGDataSet.InternalClose;
begin
  inherited;
  Self.BindFields(False);
  FAberto := False;

end;

procedure TRRPGDataSet.InternalDelete;
begin
  inherited;
  Self.RemoverRegistroDataSet();
end;

procedure TRRPGDataSet.InternalInitFieldDefs;
var
  i: Integer;
  Def: TFieldDef;
  F: TField;
begin
  inherited;
  Self.CountCamposFisicos := 0;
  Self.FieldDefs.BeginUpdate;

  try
    Self.FieldDefs.Clear;

    for i := 0 to Self.Fields.Count - 1 do
    begin
      F := Self.Fields[i];

      if F.FieldKind <> fkData then
        Continue;

      Def := Self.FieldDefs.AddFieldDef();

      Def.Name := F.FieldName;
      Def.DataType := F.DataType;
      Def.Size := F.Size;
      Def.Precision := 18;
      Self.CountCamposFisicos := Self.CountCamposFisicos + 1;
    end;
  finally
    Self.FieldDefs.EndUpdate;
  end;
end;

procedure TRRPGDataSet.InternalOpen;
begin
  inherited;
  FAberto := True;
  Self.InternalInitFieldDefs();
  Self.BindFields(True);
end;

procedure TRRPGDataSet.InternalPost;
begin
  inherited;

  if Self.State = dsEdit then
    Self.CopiarRegistroEmEdicaoParaBuffer()
  else
    Self.CopiarRegistroEmInsercaoParaBuffer();
end;

function TRRPGDataSet.InternoAlimentarProximoRecordBuffer(
  var Buf: TBufferRecord): Boolean;
begin
  Result := False;
  // No existe um cursor. Est tudo na memria
  // Se fechar o DataSet, todo o dado  perdido
end;

function TRRPGDataSet.InternoGetFieldCountForBuffer: Integer;
begin
  Result := Self.CountCamposFisicos;
end;

function TRRPGDataSet.IsCursorOpen: Boolean;
begin
  Result := FAberto;
end;

procedure TRRPGDataSet.LimparDataSet;
begin
  Self.Active := False;   // Reabrir o dataset causa uma limpeza interna do buffer
  Self.Active := True;
end;

procedure TRRPGDataSet.NotificarAlteracaoDataSet;
var
  Int: IDelphiRRPGOwnerDatasets;
begin
  if Self.Owner <> nil then
    if Self.Owner.GetInterface(IDelphiRRPGOwnerDatasets, Int) then
      Int.NotificarDataSetMudou(Self);
end;

procedure TRRPGDataSet.AbrirNormal;
begin
  Self.Active := False;
  Self.Active := True;
end;

procedure TRRPGDataSet.SetSomenteLeitura(const Value: Boolean);
begin
  if FSomenteLeitura <> Value then
  begin
    FSomenteLeitura := Value;

    if Self.State in [dsEdit, dsInsert] then
      Self.Cancel;

    Self.DataEvent(deUpdateState, 0);
  end;
end;

procedure TRRPGDataSet.DoBeforeInsert;
begin
  if FSomenteLeitura then
    raise ERRPGSomenteLeitura.Create('');

  inherited;    
end;

procedure TRRPGDataSet.DoBeforeEdit;
begin
  if FSomenteLeitura then
    raise ERRPGSomenteLeitura.Create('');

  inherited;
end;

procedure TRRPGDataSet.DoAfterCancel;
begin
  inherited;
  Self.NotificarAlteracaoDataSet();
end;

procedure TRRPGDataSet.DoAfterDelete;
begin
  inherited;
  Self.NotificarAlteracaoDataSet();
end;

procedure TRRPGDataSet.DoAfterEdit;
begin
  inherited;
  Self.NotificarAlteracaoDataSet();
end;

procedure TRRPGDataSet.DoAfterInsert;
begin
  inherited;
  Self.NotificarAlteracaoDataSet();
end;

procedure TRRPGDataSet.DoBeforeDelete;
begin
  if FSomenteLeitura then
    raise ERRPGSomenteLeitura.Create('');
    
  inherited;
end;

end.

