unit RRPGDataSetUtil;

interface

uses
  DB, Variants, FMTBcd, SysUtils;

{$I RRPGDataset.inc}

{$IFNDEF USEARRAYBUFFER}   // 24 = DELPHI XE3
type
  TValueBuffer = Pointer;
{$ENDIF}

type
  TBufferHelper = class
  public
    {$IFDEF USEARRAYBUFFER}
    class function ConverterBufferTo<T>(const Buffer: TValueBuffer): T;
    class procedure ValueToBuffer<T>(var Buffer: TValueBuffer; const Valor: T);
    class procedure CopyBytesToBuffer(var Buffer: TValueBuffer; const Src: TBytes);
    class procedure CopyValueBufferToFixedSizeValueBuffer(Dest: TValueBuffer; Src: TValueBuffer);
    {$ELSE}
    class function ConverterBufferTo<T>(const Buffer): T;
    class procedure ValueToBuffer<T>(var Buffer; const Valor: T);
    {$ENDIF}
  end;

function FieldTypeToVariantType(FieldType: TFieldType): Integer;


{$IFDEF USEARRAYBUFFER}
procedure LerVariantDeBuffer(var V: Variant; const Buffer: TValueBuffer; const Tamanho: Integer;
  Field: TField);
function EscreverVariantParaBuffer(V: Variant; var Buffer: TValueBuffer; var Tamanho: Integer; Field: TField): Boolean;
{$ELSE}
procedure LerVariantDeBuffer(var V: Variant; const Buffer; const Tamanho: Integer;
  Field: TField);
function EscreverVariantParaBuffer(V: Variant; var Buffer; var Tamanho: Integer;
  Field: TField): Boolean;
{$ENDIF}

function ConverterVariant(V: Variant; Para: Integer): Variant;

implementation

{*******************************************************************************
*
*   ConverterVariant
*
*   Sumrio: Converte um Variant de um tipo para outro
*
*   @param Para
*     O tipo de variant desejado.. Exemplos:
*     varInteger, varString, varSmallint etc..
*
*   Autor: ALC
*   Data: 06/09/09
*}

function ConverterVariant(V: Variant; Para: Integer): Variant;
begin
  if V <> Null then
  begin
    if VarType(V) = Para then
      Result := V
    else
      Result := VarAsType(V, Para);
  end else
    Result := Null;
end;

{*******************************************************************************
*
*   FieldTypeToVariantType
*
*   Sumrio: Converte um FieldType para um tipo de Variant
*
*   Autor: ALC
*   Data: 06/09/09
*}

function FieldTypeToVariantType(FieldType: TFieldType): Integer;
begin
  case FieldType of
    ftString, ftBlob: Result := varString;
    ftWideString: Result := varUString;
    ftInteger: Result := varInteger;
    ftSmallint: Result := varSmallint;
    ftWord: Result := varWord;
    ftBoolean: Result := varBoolean;
    ftFloat: Result := varDouble;
    ftCurrency: Result := varCurrency;
    ftBCD: Result := varCurrency;
    ftDate, ftTime, ftDateTime: Result := varDate;
    ftMemo: Result := varArray;
    ftLargeint: Result := varInt64;

    else
      Result := varUnknown;
  end;
end;

{*******************************************************************************
*
*   EscreverVariantParaBuffer
*
*   Sumrio:
*     Escreve o valor interno de um Variant para um buffer, de acordo com o
*     tipo do Variant
*
*   Autor: ALC
*   Data: 06/09/09
*}

{$IFDEF USEARRAYBUFFER}
function EscreverVariantParaBuffer(V: Variant; var Buffer: TValueBuffer; var Tamanho: Integer; Field: TField): Boolean;
{$ELSE}
function EscreverVariantParaBuffer(V: Variant; var Buffer; var Tamanho: Integer; Field: TField): Boolean;
{$ENDIF}
var
  VData: TVarData;
  {$IFNDEF USEARRAYBUFFER}
  MaxTamBuf: Integer;

  procedure Mover(var Dados; Tamanho: Integer);
  var
    BufClear: PByte;
  begin
    Move(Dados, Buffer, Tamanho);

    BufClear := @Buffer;
    Inc(BufClear, Tamanho);
    FillChar(BufClear^, MaxTamBuf - Tamanho, 0);
  end;
  {$ENDIF}
begin
  Result := True;

  if VarType(V) in [varNull, varEmpty] then
    Result := False;

  {$IFNDEF USEARRAYBUFFER}
  MaxTamBuf := Tamanho;
  {$ENDIF}

  VData := TVarData(V);

  if @Buffer = nil then
    Exit;

  case VarType(V) of
    varSmallint: TBufferHelper.ValueToBuffer<SmallInt>(Buffer, VData.VSmallint);
    varInteger: TBufferHelper.ValueToBuffer<Integer>(Buffer, VData.VInteger);

    {$IFDEF USEARRAYBUFFER}
    varUString: TBufferHelper.CopyBytesToBuffer(Buffer,  TEncoding.Default.GetBytes(PChar(VData.VString)));
    varString: TBufferHelper.CopyBytesToBuffer(Buffer, TEncoding.ANSI.GetBytes(String(AnsiString(PAnsiChar(VData.VString)))));
    {$ELSE}
    varUString: Mover(VData.VString^, Length(PChar(VData.VString)) * SizeOf(Char));
    varString: Mover(VData.VString^, Length(PAnsiChar(VData.VString)) * SizeOf(AnsiChar));
    {$ENDIF}

    varDouble: TBufferHelper.ValueToBuffer<Double>(Buffer, VData.VDouble);
    varCurrency: TBufferHelper.ValueToBuffer<Currency>(Buffer, VData.VCurrency);
    varInt64: TBufferHelper.ValueToBuffer<Int64>(Buffer, VData.VInt64);
    varDate: TBufferHelper.ValueToBuffer<TDateTime>(Buffer, VData.VDate);
    varBoolean: TBufferHelper.ValueToBuffer<WordBool>(Buffer, VData.VBoolean);

    else
      Result := False;
  end;
end;

{*******************************************************************************
*
*   LerVariantDeBuffer
*
*   Sumrio: Transforma um buffer para um Variant de acordo com o tipo de dados
*    do Field
*
*   Autor: ALC
*   Data: 06/09/09
*}

{$IFDEF USEARRAYBUFFER}
class function TBufferHelper.ConverterBufferTo<T>(const Buffer: TValueBuffer): T;
begin
  Move(Buffer[0], Result, SizeOf(Result));
end;

class procedure TBufferHelper.ValueToBuffer<T>(var Buffer: TValueBuffer; const Valor: T);
begin
  if Length(Buffer) < SizeOf(Valor) then
    SetLength(Buffer, SizeOf(Valor));

  Move(Valor, Buffer[0], SizeOf(Valor));
end;

class procedure TBufferHelper.CopyBytesToBuffer(var Buffer: TValueBuffer; const Src: TBytes);
var
  QtToCopy, Restante: NativeInt;
begin
  QtToCopy := Length(Src);

  if QtToCopy > Length(Buffer) then
    QtToCopy := Length(Buffer);

  Restante := Length(Buffer) - QtToCopy;
  Move(Src[0], Buffer[0], QtToCopy);
  FillChar(Buffer[QtToCopy], Restante, 0);
end;

class procedure TBufferHelper.CopyValueBufferToFixedSizeValueBuffer(Dest: TValueBuffer; Src: TValueBuffer);
var
  QtBytesCopiar: NativeInt;
begin
  QtBytesCopiar := Length(Src);

  if QtBytesCopiar > Length(Dest) then
    QtBytesCopiar := Length(Dest);

  Move(Src[0], Dest[0], QtBytesCopiar);
end;    

{$ELSE}
class function TBufferHelper.ConverterBufferTo<T>(const Buffer): T;
begin
  Result := T(Buffer);
end;

class procedure TBufferHelper.ValueToBuffer<T>(var Buffer; const Valor: T);
begin
  T(Buffer) := Valor;
end;
{$ENDIF}

{$IFDEF USEARRAYBUFFER}
procedure LerVariantDeBuffer(var V: Variant; const Buffer: TValueBuffer; const Tamanho: Integer; Field: TField);
{$ELSE}
procedure LerVariantDeBuffer(var V: Variant; const Buffer; const Tamanho: Integer; Field: TField);
{$ENDIF}

  {$IFDEF USEARRAYBUFFER}
  procedure LerString;
  begin
    V := AnsiString(TEncoding.ANSI.GetString(Buffer));
  end;

  procedure LerWideString;
  begin
    V := String(TEncoding.Default.GetString(Buffer));
  end;

  {$ELSE}
  procedure LerString;
  var
    X: PAnsiChar;
  begin
    X := @Buffer;
    V := AnsiString(X);
  end;

  procedure LerWideString;
  var
    X: PChar;
  begin
    X := @Buffer;
    V := String(X);
  end;
  {$ENDIF}
begin
  if @Buffer = nil then
  begin
    V := Null;
    Exit;
  end;

  {$IFDEF USEARRAYBUFFER}
    if Length(Buffer) = 0 then
    begin
      V := Null;
      Exit;
    end;
  {$ENDIF}

  case Field.DataType of
    ftSmallint: V := TBufferHelper.ConverterBufferTo<Smallint>(Buffer);
    ftInteger: V := TBufferHelper.ConverterBufferTo<Integer>(Buffer);
    ftString: LerString();
    ftWideString: LerWideString();

    ftFloat: V := TBufferHelper.ConverterBufferTo<Double>(Buffer);
    ftCurrency, ftBCD: V := TBufferHelper.ConverterBufferTo<Currency>(Buffer);

    ftLargeint: V := TBufferHelper.ConverterBufferTo<Int64>(Buffer);

    ftWord: V := TBufferHelper.ConverterBufferTo<Word>(Buffer);
    ftBoolean: V := TBufferHelper.ConverterBufferTo<WordBool>(Buffer);
    ftDate, ftTime, ftDateTime: V := TBufferHelper.ConverterBufferTo<TDateTime>(Buffer);
  else
    V := Null;
  end;
end;


end.
