unit RRPGDockTree;

interface

uses
  Windows, CaptionedDockTree, Graphics, DockTabSet, SysUtils, Classes;

type
  TRRPGDockCaptionPainter = class(TDockCaptionDrawer)
  public
    function GetCloseRect(const CaptionRect: TRect): TRect;
    function CalcButtonSize(const CaptionRect: TRect): Integer;
    function GetPinRect(const CaptionRect: TRect): TRect;

    procedure DrawDockCaption(const Canvas: TCanvas;
      CaptionRect: TRect; State: TParentFormState); override;
  end;

  TRRPGDockTree = class(TCaptionedTabDockTree)
  public
    class function GetDockCaptionDrawer: TDockCaptionDrawerClass; override;
  end;

implementation

uses
  {$IF RTLVERSION >= 25}UITypes, Types, {$IFEND}
  GraphUtil, RRPGColorScheme, Controls;

const
  cSideBuffer = 4;

{ TRRPGDockTree }

class function TRRPGDockTree.GetDockCaptionDrawer: TDockCaptionDrawerClass;
begin
  Result := TRRPGDockCaptionPainter;
end;

{ TRRPGDockCaptionPainter }

function TRRPGDockCaptionPainter.CalcButtonSize(
  const CaptionRect: TRect): Integer;
const
  cButtonBuffer = 8;
begin
  Result := CaptionRect.Bottom - CaptionRect.Top - cButtonBuffer;
end;

procedure TRRPGDockCaptionPainter.DrawDockCaption(const Canvas: TCanvas;
  CaptionRect: TRect; State: TParentFormState);

  procedure PaintCloseX(const LeftTip, TopTip: Integer);
  begin
    Canvas.Pen.Color := EsquemaAtual.CorLinhaShadowDockPainter2;
    Canvas.Pen.Style := psSolid;
    Canvas.Pen.Width := 1;

    // Left side
    Canvas.MoveTo(LeftTip + 0, TopTip + 0);
    Canvas.LineTo(LeftTip + 0, TopTip + 1);
    Canvas.LineTo(LeftTip + 1, TopTip + 2);
    Canvas.LineTo(LeftTip + 2, TopTip + 3);
    Canvas.LineTo(LeftTip + 1, TopTip + 4);
    Canvas.LineTo(LeftTip + 0, TopTip + 5);
    Canvas.LineTo(LeftTip + 0, TopTip + 7);

    // top
    Canvas.MoveTo(LeftTip + 1, TopTip - 1);
    Canvas.LineTo(LeftTip + 2, TopTip - 1);
    Canvas.LineTo(LeftTip + 3, TopTip + 0);
    Canvas.LineTo(LeftTip + 4, TopTip + 1);
    Canvas.LineTo(LeftTip + 5, TopTip + 0);
    Canvas.LineTo(LeftTip + 6, TopTip - 1);
    Canvas.LineTo(LeftTip + 8, TopTip - 1);

    // right side
    Canvas.MoveTo(LeftTip + 8, TopTip + 0);
    Canvas.LineTo(LeftTip + 8, TopTip + 1);
    Canvas.LineTo(LeftTip + 7, TopTip + 2);
    Canvas.LineTo(LeftTip + 6, TopTip + 3);
    Canvas.LineTo(LeftTip + 7, TopTip + 4);
    Canvas.LineTo(LeftTip + 8, TopTip + 5);
    Canvas.LineTo(LeftTip + 8, TopTip + 7);

    // bottom
    Canvas.MoveTo(LeftTip + 1, TopTip + 7);
    Canvas.LineTo(LeftTip + 2, TopTip + 7);
    Canvas.LineTo(LeftTip + 3, TopTip + 6);
    Canvas.LineTo(LeftTip + 4, TopTip + 5);
    Canvas.LineTo(LeftTip + 5, TopTip + 6);
    Canvas.LineTo(LeftTip + 6, TopTip + 7);
    Canvas.LineTo(LeftTip + 8, TopTip + 7);

    // Fill
    Canvas.Brush.Color := EsquemaAtual.CorWindow;
    Canvas.FloodFill(LeftTip + 3, TopTip + 1, Canvas.Pixels[LeftTip + 3, TopTip + 1], fsSurface);
  end;

  procedure PaintPin(const LeftTip, TopTip: Integer);
  var
    Left, Top: Integer;
  begin
    Canvas.Pen.Color := Canvas.Font.Color;
    Canvas.Pen.Style := psSolid;
    Canvas.Pen.Width := 1;

    Canvas.Pen.Color := EsquemaAtual.CorLinhaShadowDockPainter3;

    if Self.DockCaptionPinButton = dcpbDown then
    begin
      Top := TopTip + 1;
      Left := LeftTip;
      { Draw the top box }
      Canvas.MoveTo(Left + 1, Top + 4);
      Canvas.LineTo(Left + 1, Top);
      Canvas.LineTo(Left + 5, Top);
      Canvas.LineTo(Left + 5, Top + 5);
      { Draw the middle line }
      Canvas.MoveTo(Left, Top + 5);
      Canvas.LineTo(Left + 7, Top + 5);
      { Draw a depth line }
      Canvas.MoveTo(Left + 4, Top + 1);
      Canvas.LineTo(Left + 4, Top + 5);
      Canvas.MoveTo(Left + 3, Top + 6);
      Canvas.LineTo(Left + 3, Top + 6 + 4);

      Canvas.Brush.Color := EsquemaAtual.CorWindow;
      Canvas.FillRect(Rect(Left + 2, Top + 1, Left + 4, Top + 5));
    end
    else
    begin
      Top := TopTip;
      Left := LeftTip;
      { Draw the right box }
      Canvas.MoveTo(Left + 4, Top + 1);
      Canvas.LineTo(Left + 9, Top + 1);
      Canvas.LineTo(Left + 9, Top + 5);
      Canvas.LineTo(Left + 3, Top + 5);
      { Draw the middle line }
      Canvas.MoveTo(Left + 3, Top);
      Canvas.LineTo(Left + 3, Top + 7);
      { Draw a depth line }
      Canvas.MoveTo(Left + 4, Top + 4);
      Canvas.LineTo(Left + 9, Top + 4);
      Canvas.MoveTo(Left, Top + 3);
      Canvas.LineTo(Left + 3, Top + 3);

      Canvas.Brush.Color := EsquemaAtual.CorWindow;
      Canvas.FillRect(Rect(Left + 4, Top + 2, Left + 9, Top + 4));
    end;
  end;

  procedure PaintOutline(const ARect: TRect);
  begin
    Canvas.Pen.Color := EsquemaAtual.CorLinhaShadowDockPainter5;

    Canvas.FillRect(Rect(ARect.Left + 1, ARect.Top + 1,
      ARect.Right - 1, ARect.Bottom - 1));

    // Top left pixel
    Canvas.Pixels[ARect.Left + 1, ARect.Top + 1] := Canvas.Pen.Color;
    // top line
    Canvas.MoveTo(ARect.Left + 2, ARect.Top + 0);
    Canvas.LineTo(ARect.Right - 2, ARect.Top + 0);
    // top right pixel
    Canvas.Pixels[ARect.Right - 2, ARect.Top + 1] := Canvas.Pen.Color;
    // right line
    Canvas.MoveTo(ARect.Right - 1, ARect.Top + 2);
    Canvas.LineTo(ARect.Right - 1, ARect.Bottom - 2);
    // bottom right pixel
    Canvas.Pixels[ARect.Right - 2, ARect.Bottom - 2] := Canvas.Pen.Color;
    // bottom line
    Canvas.MoveTo(ARect.Left + 2, ARect.Bottom - 1);
    Canvas.LineTo(ARect.Right - 2, ARect.Bottom - 1);
    // bottom left pixel
    Canvas.Pixels[ARect.Left + 1, ARect.Bottom - 2] := Canvas.Pen.Color;
    // left line
    Canvas.MoveTo(ARect.Left + 0, ARect.Top + 2);
    Canvas.LineTo(ARect.Left + 0, ARect.Bottom - 2);

    // Lighter 3d'ish look
    // Right side
    Canvas.Pen.Color := EsquemaAtual.CorLinhaShadowDockPainter4;
    Canvas.MoveTo(ARect.Right - 4, ARect.Top + 1);
    Canvas.LineTo(ARect.Right - 3, ARect.Top + 1);
    Canvas.LineTo(ARect.Right - 3, ARect.Top + 2);
    Canvas.LineTo(ARect.Right - 2, ARect.Top + 2);
    // vert line
    Canvas.LineTo(ARect.Right - 2, ARect.Bottom - 3);
    Canvas.LineTo(ARect.Right - 3, ARect.Bottom - 3);
    Canvas.LineTo(ARect.Right - 3, ARect.Bottom - 2);
    // bottom line
    Canvas.LineTo(ARect.Left + 1, ARect.Bottom - 2);
    // last bottom left side pixel
    Canvas.Pixels[ARect.Left + 1, ARect.Bottom - 3] := Canvas.Pen.Color;
  end;

  procedure DrawCloseButton(const ARect: TRect);
  var
    OutlineRect: TRect;
    LeftTip, TopTip: Integer;
  begin
    if Self.DockCaptionPinButton = dcpbDown then
    begin
      OutlineRect.Left := ARect.Left - 1;
      OutlineRect.Top := ARect.Top - 1;
      OutlineRect.Right := ARect.Right + 2;
      OutlineRect.Bottom := ARect.Bottom + 2;
      LeftTip := ARect.Left + 2;
      TopTip := ARect.Top + 3;
    end
    else
    begin
      OutlineRect.Left := ARect.Left - 2;
      OutlineRect.Top := ARect.Top - 2;
      OutlineRect.Right := ARect.Right + 3;
      OutlineRect.Bottom := ARect.Bottom + 3;
      LeftTip := ARect.Left + 1;
      TopTip := ARect.Top + 2;
    end;

    PaintOutline(OutlineRect);
    PaintCloseX(LeftTip, TopTip);
  end;

  procedure DrawPinButton(const ARect: TRect);
  var
    OutlineRect: TRect;
    LeftTip, TopTip: Integer;
  begin
    if Self.DockCaptionPinButton = dcpbDown then
    begin
      OutlineRect.Left := ARect.Left - 1;
      OutlineRect.Top := ARect.Top - 1;
      OutlineRect.Right := ARect.Right;
      OutlineRect.Bottom := ARect.Bottom + 2;
      LeftTip := ARect.Left + 3;
      TopTip := ARect.Top + 1;
    end
    else
    begin
      OutlineRect.Left := ARect.Left - 4;
      OutlineRect.Top := ARect.Top - 3;
      OutlineRect.Right := ARect.Right - 1;
      OutlineRect.Bottom := ARect.Bottom + 3;
      LeftTip := ARect.Left - 2;
      TopTip := ARect.Top;
    end;

    PaintOutline(OutlineRect);
    PaintPin(LeftTip, TopTip);
  end;

  function RectWidth(const Rect: TRect): Integer;
  begin
    Result := Rect.Right - Rect.Left;
  end;

  procedure DrawIcon;
  var
    FormBitmap: TBitmap;
    DestBitmap: TBitmap;
    ImageSize: Integer;
    X, Y: Integer;
  begin
    if (State.Icon <> nil) and (State.Icon.HandleAllocated) then
    begin
      //if Self.DockCaptionOrientation = dcoHorizontal then
      begin
        ImageSize := CaptionRect.Bottom - CaptionRect.Top - 3;
        X := CaptionRect.Left;
        Y := CaptionRect.Top + 2;
      end;
      {else
      begin
        ImageSize := CaptionRect.Right - CaptionRect.Left - 3;
        X := CaptionRect.Left + 1;
        Y := CaptionRect.Top;
      end;}

      FormBitmap := nil;
      DestBitmap := TBitmap.Create;
      try
        FormBitmap := TBitmap.Create;
        DestBitmap.Width :=  ImageSize;
        DestBitmap.Height := ImageSize;
        DestBitmap.Canvas.Brush.Color := clFuchsia;
        DestBitmap.Canvas.FillRect(Rect(0, 0, DestBitmap.Width, DestBitmap.Height));
        FormBitmap.Width := State.Icon.Width;
        FormBitmap.Height := State.Icon.Height;
        FormBitmap.Canvas.Draw(0, 0, State.Icon);
        ScaleImage(FormBitmap, DestBitmap, DestBitmap.Width / FormBitmap.Width);

        DestBitmap.TransparentColor := DestBitmap.Canvas.Pixels[0, DestBitmap.Height - 1];
        DestBitmap.Transparent := True;

        Canvas.Draw(X, Y, DestBitmap);
      finally
        FormBitmap.Free;
        DestBitmap.Free;
      end;

      //if Self.DockCaptionOrientation = dcoHorizontal then
        CaptionRect.Left := CaptionRect.Left + 6 + ImageSize;
      //else
       // CaptionRect.Top := CaptionRect.Top + 6 + ImageSize;
    end;
  end;

var
  ShouldDrawClose: Boolean;
  CloseRect, PinRect: TRect;
begin
  //Canvas.Font.Color := State.FontColor;
  Canvas.Font.Color := EsquemaAtual.CorTextoNormal;
  Canvas.Pen.Width := 1;
  Canvas.Pen.Color := EsquemaAtual.Cores[seNormal].CorBorda2;

  //if FDockCaptionOrientation = dcoHorizontal then
  begin
    CaptionRect.Top := CaptionRect.Top + 1;

    { Fill the middle }
    if State.Focused then
      Canvas.Brush.Color := EsquemaAtual.CorWindow
    else
      Canvas.Brush.Color := EsquemaAtual.Cores[seNormal].Cor1End;

    Canvas.FillRect(Rect(CaptionRect.Left + 1, CaptionRect.Top + 1,
      CaptionRect.Right, CaptionRect.Bottom));

    { Draw a slight outline }
    //Canvas.Pen.Color := GetShadowColor(Canvas.Pen.Color, -20);

    Canvas.Pen.Color := EsquemaAtual.CorLinhaShadowDockPainter;

    with CaptionRect do
      Canvas.Polyline([Point(Left + 2, Top),
        Point(Right - 2, Top), { Top line }
        Point(Right, Top + 2), { Top right curve }
        Point(Right, Bottom - 2), { Right side line }
        Point(Right - 2, Bottom), { Bottom right curve }
        Point(Left + 2, Bottom), { Bottom line }
        Point(Left, Bottom - 2), { Bottom left curve }
        Point(Left, Top + 2), { Left side line }
        Point(Left + 3, Top)]); { Top left curve }

    { Get the close rect size/position }
    CloseRect := GetCloseRect(CaptionRect);
    { Does it have the pin button? Make some room for it, and draw it. }
    if Self.DockCaptionPinButton <> dcpbNone then
    begin
      PinRect := GetPinRect(CaptionRect);

      if Self.DockCaptionPinButton = dcpbUp then
        Inc(PinRect.Top); { Down a little further - better looks }

      DrawPinButton(PinRect);
      CaptionRect.Right := PinRect.Right - 2;
    end
    else
    begin
      { Shrink the rect to consider the close button on the right, and
        not draw text in it. }
      CaptionRect.Right := CloseRect.Right - 2;
    end;
    { Move away from the left edge a little before drawing text }
    CaptionRect.Left := CaptionRect.Left + 6;
    { Draw the icon, if found. }
    DrawIcon;
    ShouldDrawClose := CloseRect.Left >= CaptionRect.Left;
  end;
  (*else
  begin
    { Give a rounded effect }
    Canvas.MoveTo(CaptionRect.Left + 1, CaptionRect.Top + 1);
    Canvas.LineTo(CaptionRect.Right - 1, CaptionRect.Top + 1);

    { fill the middle }
    if State.Focused then
      Canvas.Brush.Color := clWindow
    else
      Canvas.Brush.Color := clBtnFace;
    Canvas.FillRect(Rect(CaptionRect.Left, CaptionRect.Top + 2,
      CaptionRect.Right, CaptionRect.Bottom));

    Canvas.Pen.Color := State.EndColor;
    Canvas.MoveTo(CaptionRect.Left + 1, CaptionRect.Bottom);
    Canvas.LineTo(CaptionRect.Right - 1, CaptionRect.Bottom);

    Canvas.Font.Orientation := 900; { 90 degrees upwards }
    { Get the close rect size/position }
    CloseRect := GetCloseRect(CaptionRect);

    { Does it have the pin button? Make some room for it, and draw it. }
    if FDockCaptionPinButton <> dcpbNone then
    begin
      PinRect := GetPinRect(CaptionRect);
      DrawPinButton(PinRect);
      CaptionRect.Top := PinRect.Bottom + 2;
    end
    else
      { Add a little spacing between the close button and the text }
      CaptionRect.Top := CloseRect.Bottom + 2;

    ShouldDrawClose := CaptionRect.Top < CaptionRect.Bottom;
    { Make the captionrect horizontal for proper clipping }
    CaptionRect.Right := CaptionRect.Left + (CaptionRect.Bottom - CaptionRect.Top - 2);
    { Position the caption starting position at most at the bottom of the
      rectangle }
    CaptionRect.Top := CaptionRect.Top + Canvas.TextWidth(State.Caption) + 2;

    if CaptionRect.Top > CaptionRect.Bottom then
      CaptionRect.Top := CaptionRect.Bottom;
  end;*)

  Canvas.Brush.Style := bsClear; { For drawing the font }
  if State.Caption <> '' then
  begin
    if State.Focused then
      Canvas.Font.Style := Canvas.Font.Style + [fsBold]
    else
      Canvas.Font.Style := Canvas.Font.Style - [fsBold];

   if ShouldDrawClose then
     CaptionRect.Right := CaptionRect.Right - (CloseRect.Right - CloseRect.Left) - 4;

    Canvas.TextRect(CaptionRect, State.Caption,
      [tfEndEllipsis, tfVerticalCenter, tfSingleLine]);
  end;

  if ShouldDrawClose then
    DrawCloseButton(CloseRect);
end;

function TRRPGDockCaptionPainter.GetCloseRect(const CaptionRect: TRect): TRect;
var
  CloseSize: Integer;
begin
  CloseSize := CalcButtonSize(CaptionRect);

//  if FDockCaptionOrientation = dcoHorizontal then
  begin
    Result.Left := CaptionRect.Right - CloseSize - cSideBuffer;
    Result.Top := CaptionRect.Top + ((CaptionRect.Bottom - CaptionRect.Top) - CloseSize) div 2;
  end;

  Result.Right := Result.Left + CloseSize;
  Result.Bottom := Result.Top + CloseSize;
end;

function TRRPGDockCaptionPainter.GetPinRect(const CaptionRect: TRect): TRect;
var
  PinSize: Integer;
begin
  PinSize := CalcButtonSize(CaptionRect);

  //if FDockCaptionOrientation = dcoHorizontal then
  begin
    Result.Left := CaptionRect.Right - 2*PinSize - 2*cSideBuffer;
    Result.Top := CaptionRect.Top + ((CaptionRect.Bottom - CaptionRect.Top) - PinSize) div 2;
  end;

  Result.Right := Result.Left + PinSize + 2;
  Result.Bottom := Result.Top + PinSize;
end;

initialization
  DefaultDockTreeClass := TRRPGDockTree;

end.
