unit RRPGModulesCreators;

interface

uses
  RRPGBaseModuleCreator, DesignIntf, ToolsApi, Classes, SysUtils, DesignEditors;

type
  TRFichaCreator = class(TRBaseModuleCreator)
  public
    function GetCreatorType: String; override;
    function NewImplSource(const ModuleIdent: String; const FormIdent: String; const AncestorIdent: String): IOTAFile;  override;
    function GetAncestorName: string; override;
  end;

  TRVisaoFichaCreator = class(TRBaseModuleCreator)
  public
    function GetCreatorType: String; override;
    function NewImplSource(const ModuleIdent: String; const FormIdent: String; const AncestorIdent: String): IOTAFile;  override;
    function GetAncestorName: string; override;
  end;


  TRFichaCreatorWizard = class(TRBaseModuleWizard, IOTAMenuWizard)
  public
    function GetMenuText: string;
    function GetComment: String; override;
    function GetName: String; override;

    procedure Execute; override;
    function GetState: TWizardState; override;
  end;

var
  FichaCreatorWizard: TRFichaCreatorWizard;

procedure Register;

implementation

procedure Register;
begin
  if FichaCreatorWizard = nil then
    FichaCreatorWizard := TRFichaCreatorWizard.Create;

  ToolsApi.RegisterPackageWizard(FichaCreatorWizard);
end;

{ TRFichaCreator }

function TRFichaCreator.GetAncestorName: string;
begin
  Result := 'RRPGFicha';
end;

function TRFichaCreator.GetCreatorType: String;
begin
  Result := sForm;
end;

function TRFichaCreator.NewImplSource(const ModuleIdent, FormIdent,
  AncestorIdent: String): IOTAFile;
var
  Conteudo: String;
begin
  Conteudo := Format(
    'unit %s;'#13#10+
    ''#13#10+
    'interface'#13#10+
    ''#13#10+
    'uses'#13#10+
    '  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,'#13#10+
    '  Dialogs, RRPGPluginFicha, RRPGBaseVisaoFicha;'#13#10+
    ''#13#10+
    'type'#13#10+
    '  T%1:s = class(TRRPGFicha)'#13#10+
    '  private'#13#10+
    '    { Private declarations }'#13#10+
    '  protected'#13#10+
    '    function GetVisaoClass: TFrmBaseVisaoFichaClass; override;'#13#10+
    '  public'#13#10+
    '    { Public declarations }'#13#10+
    '  end;'#13#10+
    ''#13#10+
    'implementation'#13#10+
    ''#13#10+
    '{$R *.dfm}'#13#10+
    ''#13#10+
    'function T%1:s.GetVisaoClass: TFrmBaseVisaoFichaClass;'#13#10+
    'begin' + #13#10+
    '  Result := nil; // Retorne uma classe TfrmVisaoFicha;'#13#10+
    'end;'#13#10+
    ''#13#10+
    'initialization'#13#10+
    '  TPluginFicha.Create(''Nome do Plugin'', ''Descricao do plugin'',' + #13#10+
    '                      ''RRPG.ID.PLUGIN'', T%1:s);'#13#10+
    ''#13#10+
    'end.', [ModuleIdent, FormIdent]);

  Result := StringToIOTAFile(Conteudo);
end;

{ TRFichaCreatorWizard }

procedure TRFichaCreatorWizard.Execute;
begin
  inherited;
  Self.ExecutarCreator(TRFichaCreator.Create);
  Self.ExecutarCreator(TRVisaoFichaCreator.Create);
end;

function TRFichaCreatorWizard.GetComment: String;
begin
  Result := 'Criar uma ficha bsica de RRPG'
end;

function TRFichaCreatorWizard.GetMenuText: string;
begin
  Result := '[RRPG] Criar um plugin de Ficha RRPG';
end;

function TRFichaCreatorWizard.GetName: String;
begin
  Result := 'RRPGModulesCreators.TRFichaCreatorWizard'
end;

function TRFichaCreatorWizard.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

{ TRVisaoFichaCreator }

function TRVisaoFichaCreator.GetAncestorName: string;
begin
  Result := 'frmVisaoFicha';
end;

function TRVisaoFichaCreator.GetCreatorType: String;
begin
  Result := sForm;
end;

function TRVisaoFichaCreator.NewImplSource(const ModuleIdent, FormIdent,
  AncestorIdent: String): IOTAFile;
var
  Conteudo: String;
begin
  Conteudo := Format(
    'unit %s;'#13#10+
    ''#13#10+
    'interface'#13#10+
    ''#13#10+
    'uses'#13#10+
    '  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,'#13#10+
    '  Dialogs, RRPGVisaoFicha, RRPGBaseVisaoFicha;'#13#10+
    ''#13#10+
    'type'#13#10+
    '  T%1:s = class(TfrmVisaoFicha)'#13#10+
    '  private'#13#10+
    '    { Private declarations }'#13#10+
    '  public'#13#10+
    '    { Public declarations }'#13#10+
    '  end;'#13#10+
    ''#13#10+
    'implementation'#13#10+
    ''#13#10+
    '{$R *.dfm}'#13#10+
    ''#13#10+
    'end.', [ModuleIdent, FormIdent]);

  Result := StringToIOTAFile(Conteudo);
end;

end.
