{*******************************************************************************}
{*                                                                             *}
{*                    RRPGModulo.pas                                           *}
{*                                                                             *}
{*    Interface de comunicao com o RRPG, usada para criar mdulos externos   *}
{*   que sero utilizados pelo RRPG                                            *}
{*                                                                             *}
{*******************************************************************************}

unit RRPGModulo;

interface

type
  IRRPGPluginFicha = interface;
  IRRPGFicha = interface;
  IRRPGFichaVisaoController = interface;

  {*******************************************************************************
  *
  *   IRRPGRegistry
  *
  *   Uma interface que representa uma pasta de um registro do Windows
  *}

  IRRPGRegistry = interface(IUnknown)
    ['{70194D70-C86A-472F-AB59-1AD58B2FDED5}']

    {GetHKey
      Retorna o Handle da Chave no Registro}
    function GetHKey: Cardinal; stdcall;

    {GetCaminhoAtual
      Retorna o caminho atual no Registro}
    function GetCaminhoAtual: PWideChar; stdcall;

    {ValorExists
      Esta funo retorna se um determinado valor (usado nas funes Ler/Escrever)
      existe}
    function ValorExiste(const Nome: PWideChar): Boolean; stdcall;

    {SubKeyExiste
      Esta funo retorna se existe um determinado SubKey dentro desta pasta }
    function SubKeyExiste(const SubKey: PWideChar): Boolean; stdcall;

    {DeletarValor
      Esta funo apaga um valor desta pasta}
    function DeletarValor(const Nome: PWideChar): Boolean; stdcall;

    {DeletarSubKey
      Esta funo apaga uma subpasta desta pasta}
    function DeletarSubKey(const SubKey: PWideChar): Boolean; stdcall;

    {AbrirSubKey
      Esta funo abre uma subpasta desta pasta, criando outro IRRPGRegistry}
    function AbrirSubKey(const SubKey: PWideChar; var Reg: IRRPGRegistry): Boolean; stdcall;

    // Funes de Leitura
    function LerCurrency(const Nome: PWideChar): Currency; stdcall;
    function LerBinaryData(const Nome: PWideChar; var Buffer; BufSize: Integer): Integer; stdcall;
    function LerBool(const Nome: PWideChar): Boolean; stdcall;
    function LerDate(const Nome: PWideChar): Double; stdcall;
    function LerDateTime(const Nome: PWideChar): Double; stdcall;
    function LerFloat(const Nome: PWideChar): Double; stdcall;
    function LerInteger(const Nome: PWideChar): Integer; stdcall;
    function LerString(const Nome: PWideChar): PWideChar; stdcall;
    function LerTime(const Nome: PWideChar): Double; stdcall;

    // Funes de Escrita
    procedure EscreverCurrency(const Nome: PWideChar; Valor: Currency); stdcall;
    procedure EscreverBinaryData(const Nome: PWideChar; var Buffer; BufSize: Integer); stdcall;
    procedure EscreverBool(const Nome: PWideChar; Valor: Boolean); stdcall;
    procedure EscreverDate(const Nome: PWideChar; Valor: Double); stdcall;
    procedure EscreverDateTime(const Nome: PWideChar; Valor: Double); stdcall;
    procedure EscreverFloat(const Nome: PWideChar; Valor: Double); stdcall;
    procedure EscreverInteger(const Nome: PWideChar; Valor: Integer); stdcall;
    procedure EscreverString(const Nome, Valor: PWideChar); stdcall;
    procedure EscreverExpandString(const Nome, Valor: PWideChar); stdcall;
    procedure EscreverTime(const Nome: PWideChar; Valor: Double); stdcall;

    property HKey: Cardinal read GetHKey;
  end;

  TPermissaoRegistro = (prRead, prReadWrite);

  {*******************************************************************************
  *
  *   IRRPGStream
  *
  *   Uma interface que representa uma poro de dados
  *}

  IRRPGStream = interface(IUnknown)
    ['{54199D9E-8F98-4FDA-86D4-0722A156D8B1}']

    {GetSize
      Retorna a quantidade de Bytes que existe no Stream}

    function GetSize: Int64; stdcall;

    {GetPosition
      Retorna a posio atual do cursor do Stream em bytes}
    function GetPosition: Int64; stdcall;

    {SetPosition
      Altera a posio atual do cursor do stream}
    procedure SetPosition(const Posicao: Int64); stdcall;

    {Read
      L a partir da posio do cursor atual do stream e escreve no buffer
      passado pelo parmetro Para. A Quantidade lida  igual ao parmetro QtBytes.

      Se conseguiu ler, o retorno  True. Caso no consiga ler, ou a quantidade de dados
      restantes no Stream for menor que QtBytes, nada sera escrito no Buffer
      e o retorno ser False}
    function Read(Para: Pointer; QtBytes: Integer): Boolean; stdcall;

    {Write
      Escreve na posio do cursor atual do stream o Buffer passado como parmetro.
      E retorna True caso conseguiu escrever, ou retorna False se no conseguir escrever}
    function Write(From: Pointer; QtBytes: Integer): Boolean; stdcall;

    property Position: Int64 read GetPosition write SetPosition;
  end;

  {******************************************************************************
  *
  *   IRRPGMemoryBuffer
  *
  *   Interface disponibilizada pelo RRPG. Interface utilitria para lhe auxiliar
  *   na hora de manter um Buffer que s escreve dados no fim do buffer e
  *   l apenas do inicio, e aps ler, o buffer lido  descartado.
  *}

  IRRPGMemoryBuffer = interface(IUnknown)
    ['{9F0B2526-3952-451E-9DB8-4441CD2EE765}']

    {AppendBuffer
      Adciona mais 1 poro de dados ao fim do buffer}
    procedure AppendBuffer(const Buffer: Pointer; QtBytes: Integer); stdcall;

    {PeekFromBuffer
      L do inicio do buffer e remove os dados do buffer.

      Retorna quantos bytes foram lidos}
    function PeekFromBuffer(const Destino: Pointer; QtBytes: Integer): Integer; stdcall;

    {GetBytesRemaining
      Retorna quantos bytes ainda restam no buffer}
    function GetBytesRemaining: Integer; stdcall;

    {LimparBuffer
      Como o prprio nome diz, limpa os dados do buffer}
    procedure LimparBuffer; stdcall;

    {SetSize
      Altera o tamanho do Buffer}
    procedure SetSize(const NovoTamanho: Integer); stdcall;
  end;

  {******************************************************************************
  *
  *   IRRPGCentral
  *
  *   Interface disponibilizada pelo RRPG.  atravs desta interface que o mdulo
  *   realiza as comunicaes mais bsicas com o RRPG.
  *}

  IRRPGCentral = interface(IUnknown)
    ['{3F41B0B4-4D16-4838-BDDD-36F49166A59A}']

    // Esta interface no possui nenhum mtodo. Faa um cast, ou utilize o mtodo
    // QueryInterface para obter as interfaces IRRPGCentralXXXXXXX, estas que realmente
    // fazem o trabalho "sujo"
  end;

  {*******************************************************************************
  *
  *   IRRPGCentralPluginFichas
  *
  *   Obtenha esta interface  partir da interface IRRPGCentral
  *
  *   Esta interface  utilizada para registrar/desregistrar plugins de fichas
  *   no RRPG
  *}

  IRRPGCentralPluginFichas = interface(IUnknown)
    ['{D81E598F-6772-4CEF-8270-7656CB074566}']

    { RegistrarPluginFicha
        Utilize este mtodo para registrar um Plugin de Fichas no RRPG.
        Voc deve implementar e apresentar a interface IRRPGPluginFicha
        para o RRPG.

        Obs: O Plugin  registrado temporariamente. Toda vez que o mdulo for Ativado,
        voc deve registrar o plugin de ficha}

    procedure RegistrarPluginFicha(const Plugin: IRRPGPluginFicha); stdcall;

    { DesregistrarPluginFicha
        Utilize este mtodo para desregistrar um plugin de ficha no RRPG previamente
        registrado pela funo RegistrarPluginFicha}

    procedure DesregistrarPluginFicha(const Plugin: IRRPGPluginFicha); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGCentralUtilitarios
  *
  *   Obtenha esta interface  partir da interface IRRPGCentral
  *
  *   Esta interface  utilizada disponibilizada para lhe auxiliar em algumas tarefas
  *   comuns.
  *}

  IRRPGCentralUtilitarios10 = interface(IUnknown)
    ['{7EFB3996-CBF7-43C5-AD4B-740DA04090DC}']

    {CriarMemoryBuffer
      Cria e retorna uma nova instncia de um MemoryBuffer}
    function CriarMemoryBuffer(out MemoryBuffer: IRRPGMemoryBuffer): Boolean; stdcall;
  end;

  TMetodoExec = procedure (const Param1, Param2: Integer) of object; stdcall;

  IRRPGCentralUtilitarios20 = interface(IRRPGCentralUtilitarios10)
    ['{7CE0C9A6-0F1E-4A0D-857F-BC81397979DC}']

    {ExecutarNaThreadPrincipal
      Executa um determinado cdigo na Thread Principal do RRPG.
      Retorna True se executou, False se no executou ou se ocorreu alguma exception}
    function ExecutarNaThreadPrincipal(const Metodo: TMetodoExec; const ValParam1, ValParam2: Integer): Boolean; stdcall;

    {ObterCaminhoRegistro
      Retorna o caminho no Registro do windows em HKEY_CURRENT_USER onde o RRPG Armazena as informaes}
    function ObterCaminhoRegistroRRPG: PWidechar; stdcall;

    {AbrirRegistro
      Cria uma interface para editar/ler uma determinada pasta do Registro do Windows}
    function AbrirRegistro(const Caminho: PWideChar; var Reg: IRRPGRegistry;
                           const Permissao: TPermissaoRegistro): Boolean; stdcall;

    {AbrirRegistroRRPG
      Cria uma interface para editar/ler o registro do RRPG}
    function AbrirRegistroRRPG(var Reg: IRRPGRegistry): Boolean; stdcall;
  end;

  IRRPGCentralUtilitarios = IRRPGCentralUtilitarios20;

  {*******************************************************************************
  *
  *   IRRPGPluginFicha   
  *
  *   Interface que o Mdulo deve implementar se desejar fazer plugins de fichas
  *   para o RRPG
  *}

  IRRPGPluginFicha = interface(IUnknown)
    ['{F58705D5-F00A-4F25-8F76-596ECD08581A}']

    { GetNome
        Retorne um nome descritivo para o sistema..
        Exemplo: Ficha de D&D 3.5}
    function GetNome: PAnsiChar; stdcall;


    { GetDescricao
        Retorne uma Descrio para o sistema ou ficha de seu plugin
        Exemplo: Sistema criado por AlyssonRPG, qualquer dvida me mande 1 e-mail}
    function GetDescricao: PAnsiChar; stdcall;


    { GetID
        Retorne um Identificador que identifica o seu plugin.
        Exemplo: RRPG.DnD35 }
    function GetID: PAnsiChar; stdcall;


    { CriarFicha
        Talvez este seja o mtodo mais importante que dever implementar.
        O RRPG ir chamar este mtodo toda vez que precisar de uma interface
        IRRPGFicha. Voc dever retornar uma nova interface IRRPGFicha}
    procedure CriarFicha(out Ficha: IRRPGFicha); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGFicha   
  *
  *   Interface implementada pelo mdulo, representa uma ficha, um personagem.
  *   Podero existir diversas IRRPGFicha no RRPG ou no Mdulo.
  *}

  IRRPGFicha = interface(IUnknown)
    ['{601A8ADF-1D58-4028-9400-97ED88CC9678}']

    {CarregarDe
      O RRPG deseja que voc carregue a ficha usando os dados que esto no
      Stream}
    procedure CarregarDe(const Stream: IRRPGStream); stdcall;

    {SalvarEm
      O RRPG deseja que voc salve a ficha no Stream passado como parmetro}
    procedure SalvarEm(const Stream: IRRPGStream); stdcall;

    {LimparFicha
      O RRPG deseja que voc resete a ficha, limpando todos os seus dados}
    procedure LimparFicha; stdcall;

    {O RRPG deseja que voc torne a ficha somente-leitura ou no, dependendo do parmetro}
    procedure SetReadOnly(const SomenteLeitura: Boolean); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGFichaVisao
  *
  *   Se uma Ficha suportar Viso, alm de implementar a interface IRRPGFicha,
  *   deve implementar esta interface tambm
  *}

  IRRPGFichaVisao = interface(IUnknown)
    ['{12BF10F9-5162-41CF-8983-C580D52E40F8}']

    {AcoplarVisaoEm
      Este mtodo  chamado pelo RRPG quando deseja que a Viso da ficha seja
      exibida em um determinado lugar.

      WindowHandle  o manipulador da janela/painel cuja a ficha deve ser acoplada.
      Pode-se utilizar o mtodo do SetParent do Windows.}
    procedure AcoplarVisaoEm(WindowHandle: Cardinal); stdcall;

    {DesacoplarVisao
      Este mtodo  chamado pelo RRPG quando quer que a Viso da ficha seja desacoplada,
      liberando os recursos alocado pelo mtodo AcoplarVisaoEm}
    procedure DesacoplarVisao; stdcall;

    {SetFichaVisaoController
      Este mtodo ser chamado pelo RRPG informando  viso da ficha qual  a
      FichaVisaoController usada para comunicar com a interface da Ficha do RRPG}
    procedure SetFichaVisaoController(const FichaVisaoController: IRRPGFichaVisaoController); stdcall;

    {RedimensionarVisao
      Este mtodo ser invocado pelo RRPG informando  viso da ficha qual tamanho
      ela deve assumir}

    procedure RedimensionarVisao(const Largura, Altura: Integer); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGMiniVisaoFichaController
  *
  *   Esta interface  implementada pelo RRPG e ser entregue  interface IRRPGMiniVisaoFicha
  *   assim que for criada.  o meio de comunicao entre uma Mini-Viso e o RRPG
  *}

  IRRPGMiniVisaoFichaController = interface(IUnknown)
    ['{E3D51086-3D87-4FAA-B2FD-FEDA026BAC34}']

    {NotificarTamanhoDesejavel
        Utilize este mtodo informando ao RRPG qual  o tamanho desejavel para
        exibir esta Mini-Viso. O RRPG tentar chegar no valor mais prximo do
        desejvel}
    procedure NotificarTamanhoDesejavel(const Largura, Altura: Integer); stdcall;

    {SalvarFicha
      Notifica ao RRPG que voc deseja gravar a ficha no RRPG. til para algumas
      Mini-Vises que alteram valores da ficha.

      Retorna True se pode salvar a ficha}
    function SalvarFicha: Boolean; stdcall;

    {ReverterFicha
      Notifca ao RRPG que voc deseja desfazer as alteraes feitas na ficha,
      recarregando os dados de acordo com a ficha que est no servidor RRPG}
    function ReverterFicha: Boolean; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGMiniVisaoFicha
  *
  *   Esta interface deve ser implementada pelo Mdulo, e representa uma Mini Viso
  *   de uma determinada Ficha
  *}

  IRRPGMiniVisaoFicha = interface(IUnknown)
    ['{225E21E8-21B6-4B3C-9FFA-A07A2F798CCF}']

    {SetMiniVisaoController
        Este mtodo ser invocado pelo RRPG informado qual  a interface
        MiniVisaoController para esta Mini-Viso}        
    procedure SetMiniVisaoController(const MiniVisaoController: IRRPGMiniVisaoFichaController); stdcall;

    {AcoplarEm
        O RRPG deseja que voc acople a mini-viso na janela passada como parmetro}
    procedure AcoplarEm(WindowHandle: Cardinal); stdcall;

    {Desacoplar
      O RRPG deseja que voc desacople a mini-viso que foi previamente acoplada
      no mtodo "AcoplarEm"}
    procedure Desacoplar; stdcall;

    {NotificarTamanhoReal
        O RRPG est te notificando qual  o tamanho real da mini-viso}
    procedure NotificarTamanhoReal(const Largura, Altura: Integer); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGFichaSuporteMiniVisao
  *
  *   Se a instncia da ficha (IRRPGFicha) suportar Mini Viso da ficha, ela
  *   deve implementar esta interface tambm
  *}

  IRRPGFichaSuporteMiniVisao = interface(IUnknown)
    ['{23BA41C9-BF24-40CB-AC09-6A3EFAAEA7D6}']

    {CriarMiniVisao
        Este mtodo ser chamado pelo RRPG quando ele precisar de uma Mini-Viso
        para esta Ficha.

        Este mtodo deve retornar uma nova MiniVisoFicha, podendo existir vrias}
    procedure CriarMiniVisao(out MiniVisao: IRRPGMiniVisaoFicha); stdcall;
  end;

  {*****************************************************************************
  *
  *   IRRPGFichaVisaoController
  *
  *   Interface disponibilizada pelo RRPG.
  *   Atravs dela voc poder se comunicar com a janela onde se encontra a
  *   viso da ficha 
  *}

  IRRPGFichaVisaoController = interface(IUnknown)
    ['{31E111CC-AD59-4150-8F66-7A1E8F68F927}']

    {NotificarAlteracao
      Notifica ao RRPG que houve alterao na Ficha}
    procedure NotificarAlteracao; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGModulo
  *
  *   Interface que o Mdulo deve implementar. Deve haver somente 1 interface desta
  *   por DLL/Mdulo.
  *}

  IRRPGModulo = interface(IUnknown)
    ['{CADEAC2E-164E-4413-AD3E-792FDB99336C}']

    { SetRRPGCentral:
        Este mtodo  invocado peo RRPG, informando ao mdulo a interface RRPGCentral
        de comunicao com o RRPG}
        
    procedure SetRRPGCentral(const RRPGCentral: IRRPGCentral); stdcall;

    { Ativar
        Este mtodo  chamado pelo RRPG. Ao ser chamado, o mdulo deve realizar todo
        o trabalho de ativao que precisa}

    procedure Ativar; stdcall;

    { Desativar
        Este mtodo  chamado pelo RRPG. Ao ser chamado, o mdulo deve fechar/liberar
        todos os recursos alocados}
    procedure Desativar; stdcall;
  end;


{O Mdulo/DLL deve exportar uma funo assim:

  procedure EntryPointRRPGModulo(out Modulo: IRRPGModulo); stdcall;

  o nome da funo deve ser EntryPointRRPGModulo (note o CaseSensitive)
}

implementation

end.
