unit RRPGModuloSom;

interface

uses
  RRPGModulo;

type
  TMixPacket = packed record
    Buffer: Pointer;
    TamanhoBuffer: Integer;
    Volume: Single;  // Valor entre 0 e 1 onde 0.0 = Sem volume, 1.0 = Volume mximo
  end;

  PMixPacket = ^TMixPacket;

  TRRPGJitterPacket = record
    Buffer: Pointer;          // Aponta para o buffer que contm o audio
    TamanhoBuffer: Integer;   // Qual  o tamanho mximo (em Bytes) do buffer do packet
    TamanhoRecorded: Integer; // Qual  o tamanho (em Bytes) do audio que est em buffer
    Timestamp: Cardinal ;     // Qual  o Timestamp em Milisegundos que representa este pacote de audio. Usado para reordenar os pacotes fora de ordem e calcular o jitter
    Span: Cardinal;           // Qual  o tamanho em MILISEGUNDOS deste pacote de audio 
  end;

  TRRPGJitterPacketEx = record
    Buffer: Pointer;          // Aponta para o buffer que contm o audio
    TamanhoBuffer: Integer;   // Qual  o tamanho mximo (em Bytes) do buffer do packet
    TamanhoRecorded: Integer; // Qual  o tamanho (em Bytes) do audio que est em buffer
    Timestamp: Double;      // Qual  o Timestamp em Milisegundos que representa este pacote de audio. Usado para reordenar os pacotes fora de ordem e calcular o jitter
    Span: Double;           // Qual  o tamanho em MILISEGUNDOS deste pacote de audio
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundMixer
  *
  *   Um "objeto" mixer do RRPG, para auxiliar a mixagem de 2 streams de audio
  *}

  IRRPGSoundMixer = interface(IUnknown)
    ['{0A88792A-0213-4216-BD51-E17F9F63DCBA}']

    {Mixar
      Chame esta funo para mixar 2 ou mais Streams de Audio.

      O Parmetro Entradas aponta para o primeiro elemento de um Array de TMixPacket
      Saida aponta para o TMixPacket resultado final, e o buffer deve ter sido
      alocado pela funo "caller"

      Retorna true se conseguiu mixar}
    function Mixar(Saida: PMixPacket; NumeroEntradas: Integer; Entradas: PMixPacket): Boolean; stdcall;

    {SelfMixar
      Chame esta funo para mixar 2 ou mais Streams de Audio.

      O Parmetro Entradas aponta para o primeiro elemento de um Array de TMixPacket
      A sada  gravada na primeira "Entrada" do array de Entradas
      Retorna true se conseguiu mixar}
    function SelfMixar(NumeroEntradas: Integer; Entradas: PMixPacket): Boolean; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundConverter
  *
  *   Interface utilitria que o RRPG oferece para lhe auxiliar na converso entre
  *   formatos de som PCM
  *}

  IRRPGSoundConverter = interface(IUnknown)
    ['{1F6754FB-76F0-4CCB-BEBB-4886CA359BD2}']

    {Converter
      Converte um pacote de som entre um formato PCM e outro.

      InBuffer - Aponta para o buffer de entrada
      InTamanho - Ao chamar deve conter o tamanho do buffer InBuffer
                  No retorno contm a quantidade de bytes de InBuffer que foi convertido}

    procedure Converter(InBuffer: Pointer; var InTamanho: Integer); stdcall;

    {GetConvertidoSize
      Retorna quantos bytes existem no buffer convertido}
    function GetConvertidoSize: Integer; stdcall;

    {LerConvertido
      L uma poro de audio do Buffer Convertido e retorna a quantidade de bytes
      realmente lidos}
    function LerConvertido(Destino: Pointer; QtBytes: Integer): Integer; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundConverter
  *
  *   Interface utilitria que o RRPG oferece para lhe auxiliar na converso entre
  *   formatos de som PCM. Para obter esta interface, execute QueryInterface em uma
  *   interface IRRPGSoundConverter
  *}

  IRRPGSoundConverter2 = interface(IRRPGSoundConverter)
    ['{2C1B4CE7-6103-4D6D-B56C-770DDA221AB0}']

    {GetPtrToConvertido
      Esta funo retorna o endereo de memria que esto os dados convertidos}
    function GetPtrToConvertido: Pointer; stdcall;

    {Limpar
      Esta funo limpa os dados convertidos}
    procedure Limpar; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundJitterBuffer
  *
  *   Interface provida pelo RRPG para lhe auxiliar em componentes que precisam
  *   de Jitter Buffer
  *
  *   Jitter Buffer  uma tcnica para enfretar Jiiter (veja a definio na internet)
  *   durante as transmisses de audio.
  *
  *   Executa automaticamente a reordenao de pacotes, e descarte de pacotes perdidos
  *   em caso de transmisso com possibilidade de perda (como o UDP)
  *}

  IRRPGSoundJitterBuffer = interface(IUnknown)
    ['{49D749C9-2574-4BF0-829B-8AEA8D6B3465}']

    {Put
      Copia e adiciona o buffer ao jitter buffer, realizando reordenao em caso
      de pacotes fora da ordem, descarta pacotes se for causar LAG na transmisso.

      Aps a chamada deste mtodo, o buffer do Packet pode ser destrudo ou usado
      em outros pontos pois o JitterBuffer faz uma cpia do buffer}
    procedure Put(var Packet: TRRPGJitterPacket); stdcall;

    {Get
      Retorna um packet do buffer que deve ser enviado  placa de som.

      Caso retorne FALSE, no foi retornando nenhum audio, e no precisa tocar nada.

      Observao: Voc deve fornecer o Buffer e preencher a propriedade TamanhoBuffer.
                  O Jitter Buffer ir preencher o restante das propriedades

      Observao 2: Se o retorno for TRUE, alm de ter que tocar o AUDIO, voc
                    deve chamar o mtodo "TocouUmPacket" assim que o audio terminar de tocar.}
    function Get(var Packet: TRRPGJitterPacket): Boolean; stdcall;

    {TocouUmPacket
      Chame este mtodo sempre que um pacote retornado pela funo "Get" foi tocada.

      OBS:  mandatrio chamar este mtodo.
      OBS2: No chame assim que o packet foi enfileirado na placa de som.
            Chame apenas depois que o pacote terminou de ser tocado}
    procedure TocouUmPacket; stdcall;

    {RealizarExecucoesInternas
      Chame este mtodo constantemente, sem necessidade de sincronizar o tempo entre
      as chamadas.

      Esta funo  chamada constantemente para que o Jitter Buffer se adapte s condies
      reais da transmisso}
    procedure RealizarExecucoesInternas; stdcall;

    {SetBaseDelay
      Configura o Delay bsico do Jitter Buffer

      BaseDelay - Delay Bsico em Milisegundos}
    procedure SetBaseDelay(const BaseDelay: Integer); stdcall;
  end;

  TOpcaoJitterBufferEx = (ojbAutoControleBufferTocado,  // Opo BOOLEANA (1 = True, 0 = False). Indica se o jitter far controle automtico dos buffers que forem tocados. Default = Ativado
                          objMaxBufferToPlay,           // Opo Cardinal (milisegundos). Representa o tamanho ideal em milisegundos do audio buffered-to-play. 0 = tamanho automatico. Padro = 0
                          objForceDWORD = $FFFFFFF);

  TValorOpcaoJitterBufferEx = Cardinal;

  {*******************************************************************************
  *
  *   IRRPGSoundJitterBufferEx
  *
  *   Interface provida pelo RRPG para lhe auxiliar em componentes que precisam
  *   de Jitter Buffer
  *
  *   Jitter Buffer  uma tcnica para enfretar Jiiter (veja a definio na internet)
  *   durante as transmisses de audio.
  *
  *   Executa automaticamente a reordenao de pacotes, e descarte de pacotes perdidos
  *   em caso de transmisso com possibilidade de perda (como o UDP)
  *}

  IRRPGSoundJitterBufferEx = interface(IUnknown)
    ['{EC28451D-05E3-4A73-8275-631ABD585B14}']

    {Put
      Copia e adiciona o buffer ao jitter buffer, realizando reordenao em caso
      de pacotes fora da ordem, descarta pacotes se for causar LAG na transmisso.

      Aps a chamada deste mtodo, o buffer do Packet pode ser destrudo ou usado
      em outros pontos pois o JitterBuffer faz uma cpia do buffer}
    procedure Put(var Packet: TRRPGJitterPacketEx); stdcall;

    {Get
      Retorna um packet do buffer que deve ser enviado  placa de som.

      Caso retorne FALSE, no foi retornando nenhum audio, e no precisa tocar nada.

      Observao: Voc deve fornecer o Buffer e preencher a propriedade TamanhoBuffer.
                  O Jitter Buffer ir preencher o restante das propriedades

      Observao 2: Se o retorno for TRUE, voc deve tocar o audio retornado}
    function Get(var Packet: TRRPGJitterPacketEx): Boolean; stdcall;

    {TocouUmBufferMS
      Esta funo s faz sentido se a opo ojbAutoControleBufferTocado for desativada.
      Chame este mtodo sempre que um pacote retornado pela funo "Get" for realmente tocado.

      OBS:  mandatrio chamar este mtodo se a opcao ojbAutoControleBufferTocado estiver desativada.
      OBS2: No chame assim que o packet foi enfileirado na placa de som.
            Chame apenas depois que o pacote terminou de ser tocado}
    procedure TocouUmBufferMS(const TempoTocado: Double); stdcall;

    {TocouUmBufferBytes
      Esta funo s faz sentido se a opo ojbAutoControleBufferTocado for desativada.

      Chame este mtodo sempre que um pacote retornado pela funo "Get" for realmente tocado.

      OBS:  mandatrio chamar este mtodo se a opcao ojbAutoControleBufferTocado estiver desativada.
      OBS2: No chame assim que o packet foi enfileirado na placa de som.
            Chame apenas depois que o pacote terminou de ser tocado}
    procedure TocouUmBufferBytes(const QtBytesTocado: Integer); stdcall;

    {GetOpcao
      Esta funo retorna o valor de alguma opo do JitterBufferEx}
    function GetOpcao(const Opcao: TOpcaoJitterBufferEx): TValorOpcaoJitterBufferEx; stdcall;

    {SetOpcao
      Esta funo seta um valor  uma opo do JitterBufferEx}
    procedure SetOpcao(const Opcao: TOpcaoJitterBufferEx; const Valor: TValorOpcaoJitterBufferEx); stdcall;

    {Limpar
      Esta funo limpa o audio que est contido no JitterBuffer, ignorar os pacotes que esto
      sendo tocados, mas continua com as estatsticas }
    procedure Limpar; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundPlayerCallback
  *
  *   Implemente esta interface para fazer comunicao com IRRPGSoundPlayer
  *}

  IRRPGSoundPlayerCallback = interface(IUnknown)
    ['{D9F7043D-DB88-4B7D-9FF0-B9FBD50BE78B}']

    {OnPacketPlayed
      Este mtodo  chamado sempre que um pacote de audio (Funo EscreverSom) for tocado.}
    procedure OnPacketPlayed; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundPlayerCallbackEx
  *
  *   Implemente esta interface para fazer comunicao com IRRPGSoundPlayer
  *   Esta interface  chamada atravs do QueryInterface de uma interface IRRPGSoundPlayerCallback
  *}

  IRRPGSoundPlayerCallbackEx = interface(IUnknown)
    ['{F1A26CE4-A776-4B00-B4DD-DB0FD73410D7}']

    {OnTocouUmBufferBytes
      Este mtodo  chamado sempre que um pacote de audio (Funo EscreverSom) for tocado.}
    procedure OnTocouUmBufferBytes(const QtBytesTocado: Integer); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundPlayer   
  *
  *   Interface que o RRPG lhe d, para tocar sons/msicas
  *}

  IRRPGSoundPlayer = interface(IUnknown)
    ['{50E565A8-065A-41FE-939C-FAB2D6F06BEA}']

    {EscreverSom
      Chame este mtodo para escrever um buffer de som a ser tocado}
    procedure EscreverSom(Buffer: Pointer; TamanhoBuffer: Integer); stdcall;

    {Parar
      Pra a execuo do som e limpa os buffers que ainda no foram tocados
      pela placa de som}
    procedure Parar; stdcall;


    {SetCallback
      Associa um IRRPGSoundPlayerCallback para este player}
    procedure SetCallback(const Callback: IRRPGSoundPlayerCallback); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundPreprocessor
  *
  *   Interface que o RRPG lhe d, para lhe auxiliar no tratamento de um stream
  *   de audio
  *}

  IRRPGSoundPreprocessor = interface(IUnknown)
    ['{09012B5B-8971-44C2-9AC7-682ACF4DD69C}']

    {SetAutoGain
      Ativa ou desativa o controle automtico de ganho}
    procedure SetAutoGain(const Ativo: Boolean); stdcall;

    {SetAutoGainVolumeDesejado
      Configura o volume desejado ao processar o Auto-Ganho.

      1.0 = Volume mximo
      0.0 = Volume Mnimo, desligado}
    procedure SetAutoGainVolumeDesejado(const Volume: Single); stdcall;

    {SetAutoGainMaxIncrement e SetAutoGainMaxDecrement
      Ajuste a suavidade do AutoGanho, definindo o mximo de decibeis por segundo
      que o volume do audio ser ajustado por segundo}
    procedure SetAutoGainMaxIncrement(const DecibeisPorSegundo: Integer); stdcall;
    procedure SetAutoGainMaxDecrement(const DecibeisPorSegundo: Integer); stdcall;

    {SetNoiseSuppress
      Ativa ou Desativa o Supressor de rudos.}
    procedure SetNoiseSuppress(const Ativo: Boolean); stdcall;

    {ProcessarSom
      Processa um determinado Buffer em PCM

      ATENO: O Tamanho do buffer em SAMPLES deve ser mltiplo do parmetro "SamplesPorFrame"
               utilizado para criar este SoundPreProcessor}
    procedure ProcessarSom(Buffer: Pointer; Tamanho: Integer); stdcall;

    {ConverterVolume
      Escala o volume do audio. Exemplo.

      Se o parmetro Escala for:
        * 1.0 = O Som no ser afetado, o volume continuar a ser o mesmo
        * 0.5 = O som ter seu volume pela metade
        * 0.0 = O Som ser abafado
        * 2.0 = O Som ter seu volume dobrado}
    procedure EscalarVolume(Buffer: Pointer; Tamanho: Integer; const Escala: Single); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipCoderStream
  *
  *   Uma interface provida pelo RRPG para lhe auxiliar. Representa um Coder de audio
  *   usado pelo VOIP
  *}

  IRRPGVoipEncoderStream = interface(IUnknown)
    ['{6B05B878-34CA-454F-8C9E-29351AE01614}']

    {Codificar
      Utilize esta funo para codificar um pacote de audio em PCM}
    procedure Codificar(const Buffer: Pointer; Tamanho: Integer); stdcall;

    {GetSizeCodificado
      Aps uma ou vrias chamadas ao mtodo "Codificar", utilize este mtodo para
      obter o tamanho em bytes do Buffer Codificado}
    function GetSizeCodificado: Integer; stdcall;

    {LerCodificado
      Utilize este mtodo para ler e remover do Bufer Codificado pela funo "Codificar".
      Retorna a quantidade de bytes escrita no parmetro Destino}
    function LerCodificado(Destino: Pointer; Tamanho: Integer): Integer; stdcall;

    {FlushPacket
      Alguns encoders ficam aguardando mais audio antes de escrever o audio final codificado.
      O RRPG chama esta funo para codificar o audio que ja foi passado pois 
      preciso enviar via net agora.}
    procedure FlushPacket; stdcall;

    {Clonar
      Retorna uma outra instncia de VoipCoderStream com as mesmas configuraes}
    function Clonar(out VCS: IRRPGVoipEncoderStream): Boolean; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipDecoderStream
  *
  *   Uma interface provida pelo RRPG para lhe auxiliar. Representa um Decoder de audio
  *   usado pelo VOIP
  *}

  IRRPGVoipDecoderStream = interface(IUnknown)
    ['{4FB4CB14-BF5C-4813-A478-DA5DCD3F339D}']

    {Decodificar
      Decodifica um pacote de audio previamente codificado}
    procedure Decodificar(const Buffer: Pointer; Tamanho: Integer); stdcall;

    {GetSizeDecodificado
      Aps uma ou vrias chamadas ao mtodo "Decodificar", utilize este mtodo para
      obter o tamanho em bytes do Buffer Decodificado}
    function GetSizeDecodificado: Integer; stdcall;

    {LerDecodificado
      Utilize este mtodo para ler e remover do Buffer Decodificado pela funo "Decodificar".
      Retorna a quantidade de bytes escrita em "Destino"}
    function LerDecodificado(Destino: Pointer; Tamanho: Integer): Integer; stdcall;

    {FlushPacket
      Este mtodo  chamado informando que no existem mais dados e preciso de um
      pacote de udio agora para tocar}
    procedure FlushPacket; stdcall;

    {Clonar
      Retorna uma outra instncia de VoipDecoderStream com as mesmas configuraes}
    function Clonar(out VDS: IRRPGVoipDecoderStream): Boolean; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundDecoderStream
  *
  *   Implemente esta interface. Esta interface representa um Stream de 1 unico arquivo
  *   de audio. Lembre-se, podem existir vrios SoundDecoderStreams criados
  *}

  IRRPGFileSoundDecoderStream = interface(IUnknown)
    ['{2637A0EC-CFAE-4746-8A33-A21543D2B322}']

    {GetInformacaoStream
      Retorne para o RRPG as informaes deste Stream de Audio}
    procedure GetInformacaoStream(out Frequencia, BitsPorSample, Canais: Integer); stdcall;

    {GetDuracao
      Retorne para o RRPG a durao em Milisegundos do SoundDecoderStream.

      -1 = No consigo calcular a durao}
    function GetDuracao: Integer; stdcall;

    {LerEDecodificar
      Leia, decodifique e retorne um pedao de buffer para o RRPG.

      Retorne a quantidade de bytes lidas, sendo que deve ser menor ou igual ao
      parmetro TamanhoBuffer

      O RRPG ir chamar esta funo enquanto a funo no retornar -1.
      Ao retornar -1, o RRPG entende que chegou ao fim do Buffer}
    function LerEDecodificar(Buffer: Pointer; const TamanhoBuffer: Integer): Integer; stdcall;


    {IrPara
      O RRPG deseja que voc aponte o decoder para o determinado ponto,
      onde o parmetro Ponto est em milisegundos.

      Retorne False caso no consiga realizar esta operao}
    function IrPara(const Ponto: Integer): Boolean; stdcall;

    {GetFrameSize
      Retorna o tamanho em BYTES de um nico Frame de Audio. As chamadas  funo
      LerEDecodificar deve conter Buffer com este Tamanho.

      Em caso de -1, no  possvel calcular o FrameSize}
    function GetFrameSize: Integer; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGSoundDecoder
  *
  *   Implemente esta interface para registrar um Decodificador de Som para uma
  *   nova extenso de arquivos
  *}

  IRRPGFileSoundDecoder = interface(IUnknown)
    ['{F2A25B1D-FB79-44A0-992B-955377E9BBCC}']

    {ConsegueDecodificar
        Retorne True caso consiga decodificar o arquivo passado como parmetro}
    function ConsegueDecodificar(const Arquivo: PAnsiChar): Boolean; stdcall;

    {CriarSoundDecoderStream
      Voc deve instanciar e retornar um novo SoundDecoderStream baseado no nome do arquivo;
      Deve retornar True se conseguiu criar}
    function CriarFileSoundDecoderStream(const Arquivo: PAnsiChar; out Stream: IRRPGFileSoundDecoderStream): Boolean; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipSessionDSP
  *
  *   Implemente esta interface para criar efeitos ou mixar sons ao son que
  *   ser enviado ao VOIP
  *}

  IRRPGVoipSessionDSP = interface(IUnknown)
    ['{9565E304-85C1-48D0-BE8E-BAC0C19433EA}']

    {ProcessarDSP
      O RRPG deseja que voc realize o DSP neste pacote de Audio.

      Sinalize o parmetro ENVIAR para True caso voc queira que envie o pacote,
      ou False se no quiser que seja enviado. No altere o valor de ENVIAR
      caso no queira interferir na deciso

      Se pelo menos um dos DSPs colocarem ENVIAR para True, ento o pacote
      ser enviado}
    procedure ProcessarDSP(Buffer: Pointer; Tamanho: Integer; var Enviar: Boolean); stdcall;

    {SomFinal
      O RRPG quer lhe mostrar como  o som final, depois de todos os DSPs processados, antes
      de enviar o contedo. No altere o contedo do buffer}
    procedure SomFinal(Buffer: Pointer; Tamanho: Integer); stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipSessionDSP2
  *
  *   Implemente esta interface para criar efeitos ou mixar sons ao son que
  *   ser enviado ao VOIP
  *}

  IRRPGVoipSessionDSP2 = interface(IUnknown)
    ['{B7E64FA1-7D17-450A-A949-177C0D0255F9}']

    {ProcessarDSP2
      O RRPG deseja que voc realize o DSP neste pacote de Audio.

      Sinalize o parmetro ENVIAR para True caso voc queira que envie o pacote,
      ou False se no quiser que seja enviado. No altere o valor de ENVIAR
      caso no queira interferir na deciso

      Se pelo menos um dos DSPs colocarem ENVIAR para True, ento o pacote
      ser enviado}
    procedure ProcessarDSP2(Buffer: Pointer; Tamanho: Integer; const TimestampMS: Double; var Enviar: Boolean); stdcall;
  end;


  {*******************************************************************************
  *
  *   IRRPGVoipSession
  *
  *   Esta interface  provida pelo RRPG FireCast e representa uma nica Sesso Voip
  *}

  IRRPGVoipSession = interface(IUnknown)
    ['{24DC63EC-F8C6-461F-9B7E-64F0B56799F2}']

    {GetFormatoSaida
      Retorna qual  a taxa de amostragem (em Hz)/Samples por segundo
      que o VOIP est operando localmente e enviando aos demais participantes da
      sesso voip.

      Pode ser: 8000, 16000 ou 32000.
      BitsPorSample: Sempre 16
      Canais: Sempre 1 canal}
    function GetFormatoSaida(out Frequencia: Integer): Boolean; stdcall;

    {AdicionarDSP e RemoverDSP
      Adiciona e Remove um DSP - Digital Signal Processing para que seja
      possvel adicionar efeitos antes de enviar o som para os demais participantes
      da sesso}
    procedure AdicionarDSP(const DSP: IRRPGVoipSessionDSP); stdcall;
    procedure RemoverDSP(const DSP: IRRPGVoipSessionDSP); stdcall;

    {CriarEncoderStream
      Retorna um novo VoipCoderStream muito similar ao que o VOIP Session est
      usando atualmente para codificar o audio e enviar  sesso de voz}
    function CriarEncoderStream(out Coder: IRRPGVoipEncoderStream): Boolean; stdcall;

    {CriarDecoderStream
      Retorna um novo VoipDecoderStream capaz de decodificar um audio codificado
      por um CoderStream retornado pela funo "CriarCoderStream"}
    function CriarDecoderStream(out Decoder: IRRPGVoipDecoderStream): Boolean; stdcall;

    {GetFrameSize
      Retorna em Milisegundos, o tamanho do FrameSize do audio gravado localmente}
    function GetFrameSize: Integer; stdcall;

    {GetPacketSize
      Retorna em Milisegundos, o tamanho de cada Packet de audio enviado para os
      jogadores.

      1 Packet possui vrios Frames de audio dentro dele}
    function GetPacketSize: Integer; stdcall;

    {GetIDMesa
      Retorna o Identificador da Mesa cujo Voip est ativo}
    function GetIDMesa: Integer; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipSessionAttachment
  *
  *   Implemente esta interface para poder acompanhar cada sesso voip existente
  *   no RRPG. Podem existir diversas instncias desta interface.
  *
  *   ATENO: Todas as implementaes devem ser Thread-Safe
  *}

  IRRPGVoipSessionAttachment = interface(IUnknown)
    ['{23D9FC34-F16F-4B43-AF7F-637D1F8A9101}']

    {SetVoipSession
      Ser chamado pelo RRPG para voc gravar qual  o VoipSession, o seu
      meio de comunicao com a sesso VOIP atual}
    procedure SetVoipSession(const VoipSession: IRRPGVoipSession); stdcall;

    {OnRodando
      Ser chamado pelo RRPG para notificar que o VOIP est ativo e enviando}
    procedure OnRodando; stdcall;

    {OnParou
      Ser chamado pelo RRPG para notificar que o VOIP est inativo e parou de enviar}
    procedure OnParou; stdcall;

    {Inicializar
      Ser chamado pelo RRPG quando ele desejar que voc se inicialize}
    procedure Inicializar; stdcall;

    {Finalizar
      Ser chamado pelo RRPG quando ele desejar que voc finalize tudo o que
      foi previamente inicializado pela funo Inicializar..

      OBS: No se destrua! Se destrua apenas no mtodo _release}
    procedure Finalizar; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipSessionsNotify
  *
  *   Implemente esta interface para registrar um Voip Sessions Notify,
  *   podendo receber notificaes de nova sesso voip e anexar a ela
  *}

  IRRPGVoipSessionsNotify = interface(IUnknown)
    ['{F72350E1-D48F-4FAA-8B26-023916AC98BD}']

    {CriarVoipSessionAttachment
      O RRPG deseja que voc retorne uma nova instncia de um VoipSessionAttachment
      para acompanhar uma nova sesso voip}
    function CriarVoipSessionAttachment(out VSA: IRRPGVoipSessionAttachment): Boolean; stdcall;
  end;

  TVSFEncoderOpt = packed record
    Descricao: PAnsiChar;  // Descrio da Opo. Exemplos "Baixa qualidade" "Boa qualidade" "tima qualidade"
    Maximobps: Integer;   // Quantide mxima de bits por segundo que esta opo pode chegar
    InformacoesInternasVSF: Integer; // Algumas informaes internas do VSF (Voip Stream Format)
  end;

  {*******************************************************************************
  *
  *   IRRPGVoipStreamFormat
  *
  *   Implemente esta interface caso queira prover um novo CODEC para ser utilizado
  *   na sesso de voz
  *}

  IRRPGVoipStreamFormat = interface(IUnknown)
    ['{F8E661E2-ECAA-47F6-9CB3-82C929C1F9F8}']

    {GetIDVoipStreamFormat
      Retorne o ID do Voip Stream Format. Verifique a unit RRPGProtocoloVoz os
      formatos j existentes}
    function GetIDVoipStreamFormat: Byte; stdcall;

    {GetAudioSamplerate
      Retorne quantos samples/segundo (frequencia ou samplerate) de audio em hZ que este
      VoipStream opera}
    function GetAudioSamplerate: Integer; stdcall;

    {GetShortName
      Retorne um nome curto para o VoipStreamFormat.
      Exemplos: - 'Speex'
                  'Speex Banda-Larga'
                  'MP3 16khz'}
    function GetShortName: PAnsiChar; stdcall;

    {GetDescricao
      Retorne uma descrio maior para o VoipStreamFormat.
      Exemplos: - 'A melhor escolha caso sua banda no seja larga e ir utilizar apenas VOZ'
                  'A melhor escolha caso sua banda seja larga e ir utilizar apenas VOZ'
                  'Se for reproduzir msicas e sons, MP3  a melhor soluo'}
    function GetDescricao: PAnsiChar; stdcall;

    {GetCoderConfigsCount
      Retorne a quantidade de opes de codificao de audio que voc pode disponibilizar
      para o usurio escolher}
    function GetEncoderOptCount: Integer; stdcall;

    {GetEncoderOpt
      Retorne os dados de uma opo de codificao de adio que voc pode disponibilizar
      para o usurio escolher.

      Retorne True caso consiga retornar.

      OBS: O parmetro Indice comea do 0 e vai ate "GetEncoderOptCount" - 1}
    function GetEncoderOpt(const Indice: Integer; var Opt: TVSFEncoderOpt): Boolean; stdcall;

    {CriarEncoderStream
      Cria e retorna uma nova instncia de IRRPGVoipEncoderStream com as configuraes
      de acordo com a Opo passada no parmetro IndiceEncoderOpt.

      Retorna True se conseguir}
    function CriarEncoderStream(const IndiceEncoderOpt: Integer; out Encoder: IRRPGVoipEncoderStream): Boolean; stdcall;

    {CriarDecoderStream
      Cria e retorna uma nova instncia de IRRPGVoipDecoderStream capaz de decodificar
      um Stream codificado pelo IRRPGVoipEncoderStream criado pela funo CriarEncoderStream.

      Retorna True se conseguir}
    function CriarDecoderStream(out Decoder: IRRPGVoipDecoderStream): Boolean; stdcall;

    {GetFrameSize
      Retorne em MILISEGUNDOS o tamanho de cada frame de audio, pois assim o RRPG
      poder disponobilizar pacotes de audio cujo tamanho seja mltiplo de FrameSize}
    function GetFrameSize: Integer; stdcall;
  end;

  {*******************************************************************************
  *
  *   IRRPGCentralSons
  *
  *   Obtenha esta interface  partir da interface IRRPGCentral
  *
  *   Esta interface  usada para registrar/desregistrar plugins de Sons
  *}

  IRRPGCentralSons10 = interface(IUnknown)
    ['{CAD21B85-507F-4A2F-97B7-CC8234854ECD}']

    {CriarSoundMixer
      Esta funo retorna um SoundMixer para lhe auxiliar.

      O SoundMixer mixa sons: 1 Canal, 16bits por sample}
    procedure CriarSoundMixer(out Mixer: IRRPGSoundMixer); stdcall;

    {CriarSoundConverter
      Esta funo retorna um SoundConverter para lhe auxiliar}
    function CriarSoundConverter(const InFrequencia, InBitsPorSample, InCanais,
                                  OutFrequencia, OutBitsPorSample, OutCanais: Integer;
                                  out Converter: IRRPGSoundConverter): Boolean; stdcall;

    {CriarSoundPlayer
      Esta funo retorna um SoundPlayer para lhe auxiliar a tocar sons.

      Retorna TRUE se criou ou false se no criou}
    function CriarSoundPlayer(const Frequencia, BitsPorSample, Canais: Integer;
                              out Player: IRRPGSoundPlayer): Boolean; stdcall;

    {CriarSoundPreprocessor
      Esta funo retorna um SoundPreprocessor para a lhe auxiliar no tratamento de
      audio.

      SamplesPorFrame - O SoundProcessor processa o som Frame a Frame.
                        Especifique o nmero de samples que existe em cada Frame.

                        Dica: Use um valor de Samples que represente entre 10 a 20ms

      Especifique a Taxa de Amostragem do som.
      O Som deve ser MONO (1 canal apenas) e deve ter 16bits por sample}
    function CriarSoundPreprocessor(const SamplesPorFrame, Frequencia: Integer; out Preprocessor: IRRPGSoundPreprocessor): Boolean; stdcall;

    {CriarSoundJitterBuffer
      Cria e retorna um JitterBuffer.

      TempoCadaPacket - Representa o tamanho em MILISEGUNDOS de cada pacote de audio
      TempoMaxAudioBufferedtoPlay - Representa o tempo mximo (em milisegundos) de pacotes de audio que
                                 ser enfileirado na placa de som..

                                 Um valor baixo demais pode fazer com que seja reproduzido um som "picado",
                                 um valor alto pode causar um alto lag.

                                 Recomendvel: aproximadamente 200ms

      FrequenciaAudio - Quantos samples por segundo o audio possui? Medido em Hertz
      BitsPorSample - Quantos bits possui cada "Sample" do udio?
      Canais - Nmero de canais que o SOM possui
                                 }

    function CriarSoundJitterBuffer(const TempoCadaPacket,
                                          TempoMaxAudioBufferedtoPlay,
                                          FrequenciaAudio,
                                          BitsPorSample,
                                          Canais: Integer;
                                    out JitterBuffer: IRRPGSoundJitterBuffer): Boolean; stdcall; deprecated 'Utilize a funo CriarSoundJitterBufferEx';
    {RegistrarDecoder e DesregistrarDecoder
      Registra e desregistra um decodificador de arquivo de som no RRPG}
    procedure RegistrarFileDecoder(const Decoder: IRRPGFileSoundDecoder); stdcall;
    procedure DesregistrarFileDecoder(const Decoder: IRRPGFileSoundDecoder); stdcall;

    {CriarFileDecoderStream
      Cria e retorna um FileDecoderStream capaz de decodificar o arquivo passado como parmetro.

      Retorna False se nenhum FileDecoderStream consegue decodificar}
    function CriarFileDecoderStream(const Arquivo: PAnsiChar; out FDS: IRRPGFileSoundDecoderStream): Boolean; stdcall;

    {RegistrarVoipSessionsNotify e DesregistrarVoipSessionNotify
      Registra e desregistra um Voip Sessions Notify}
    procedure RegistrarVoipSessionsNotify(const VSN: IRRPGVoipSessionsNotify); stdcall;
    procedure DesregistrarVoipSessionNotify(const VSN: IRRPGVoipSessionsNotify); stdcall;

    {RegistrarVoipStreamFormat e DesregistrarVoipStreamFormat
      Registra e desregistra um Voip Stream Format}
    procedure RegistrarVoipStreamFormat(const VSF: IRRPGVoipStreamFormat); stdcall;
    procedure DesregistrarVoipStreamFormat(const VSF: IRRPGVoipStreamFormat); stdcall;

    {CountVoipStreamFormat
      Retorna a quantidade de VoipStreamFormat registrado no RRPG}
    function CountVoipStreamFormat: Integer; stdcall;

    {GetVoipStreamFormat
      Retorna um VoipStreamFormat cujo Indice seja igual ao passado no parmetro Indice.

      Resulta TRUE se conseguiu retornar em VSF}
    function GetVoipStreamFormat(const Indice: Integer; out VSF: IRRPGVoipStreamFormat): Boolean; stdcall;

    {FindVoipStreamFormat
      Localiza e retorna (se localizado) um VoipStreamFormat cujo ID seja igual ao passado
      pelo parmetro IDVoipStreamFormat}
    function FindVoipStreamFormat(const IDVoipStreamFormat: Integer; out VSF: IRRPGVoipStreamFormat): Boolean; stdcall;
  end;

  IRRPGCentralSons20 = interface(IRRPGCentralSons10)
    ['{7AA089F6-D676-48FF-8FFB-F485F0F2BF06}']

    {CriarSoundJitterBufferEx
      Cria e retorna um JitterBufferEx.

      FrequenciaAudio - Quantos samples por segundo o audio possui? Medido em Hertz
      BitsPorSample - Quantos bits possui cada "Sample" do udio?
      Canais - Nmero de canais que o SOM possui
                                 }

    function CriarSoundJitterBufferEx(FrequenciaAudio,
                                      BitsPorSample,
                                      Canais: Integer;
                                      out JitterBuffer: IRRPGSoundJitterBufferEx): Boolean; stdcall;
  end;

  IRRPGCentralSons = IRRPGCentralSons20;

implementation

end.
