unit RRPGPageControl;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, StdCtrls, ExtCtrls, RRPGDrawUtil,
  CommCtrl;  // DBCtrls

type
  TRTabSheet = class(TTabSheet)
  private
    FCanvasDraw: TCanvas;
    FPictureIcon: TPicture;
    FTabHint: String;
    procedure SetPictureIcon(const Value: TPicture);
  protected
    procedure PaintWindow(DC: HDC); override;

    procedure DesenharEm(DC: HDC);
    procedure ConfigurarVisual; virtual;
    procedure Loaded; override;
    procedure WMPrintClient(var Message: TWMPrintClient); message WM_PRINTCLIENT;
    procedure WMEraseBkg(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure PictureChanged(Sender: TObject); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color;
    property PictureIcon: TPicture read FPictureIcon write SetPictureIcon;
    property TabHint: String read FTabHint write FTabHint;
  end;

  TRPageControl = class(TPageControl)
  private
    FCanvasDraw: TCanvas;
    FTransparentTab: Boolean;
    procedure SetTransparentTab(const Value: Boolean);
  protected
    procedure DesenharEm(DC: HDC);
    procedure DesenharTab(C: TCanvas; Indice, Altura: Integer);
    function IsMouseSobreControle(R: TRect): Boolean;
    procedure Change; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure MouseMove(Shift: TShiftState; X: Integer; Y: Integer); override;

    procedure WMEraseBkg(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure PaintWindow(DC: HDC); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure NovaPagina(const Caption: String);
    function RemoverPagina: Boolean;
  published
    property TransparentTab: Boolean read FTransparentTab write SetTransparentTab default False;
  end;


implementation

uses
  {$IF RTLVERSION >= 25}UITypes,{$IFEND}
  GraphUtil, UxTheme, RRPGUtil, Themes, RRPGColorScheme, GDIPlus, Math,
  Types;

{ TRPageControl }

procedure TRPageControl.Change;
begin
  inherited;
  Self.Invalidate;
end;

procedure TRPageControl.CMMouseLeave(var Message: TMessage);
begin
  if Self.Hint <> '' then
    Self.Hint := '';

  inherited;
end;

constructor TRPageControl.Create(AOwner: TComponent);
begin
  inherited;
  FCanvasDraw := TCanvas.Create;
end;

procedure TRPageControl.CreateParams(var Params: TCreateParams);
begin
  inherited;

{  if not (csDesigning in Self.ComponentState) then
  begin
    if FTransparentTab then
      Params.ExStyle := Params.ExStyle or WS_EX_TRANSPARENT;
  end;}
end;

procedure TRPageControl.DesenharEm(DC: HDC);
var
  C: TCanvas;
  R, R2: TRect;
  Count, i: Integer;
  AP: TTabSheet;
  TemaAtivo: Boolean;
begin
  C := FCanvasDraw;
  C.Handle := DC;

  try
    Count := SendMessage(Self.Handle, TCM_GETITEMCOUNT, 0, 0);
    AP := Self.ActivePage;

    if AP <> nil then
    begin
      TabCtrl_GetItemRect(Self.Handle, 0, R);
      R := Self.ClientRect;
      TemaAtivo := ThemeControl(Self);

      if FTransparentTab and TemaAtivo then
      begin
        GetThemeServices().DrawParentBackground(Self.Handle, DC, nil, False, R);
      end else
        begin
          C.Brush.Color := Self.Color;
          C.Brush.Style := bsSolid;
          C.Pen.Style := psClear;
          C.Rectangle(R);
        end;

      //R := Rect(AP.Left - 1, AP.Top - 1, AP.Left + AP.Width + 1, AP.Top + AP.Height + 1);
      C.Font.Assign(Self.Font);
      C.Font.Color := EsquemaAtual.Cores[seNormal].CorFonte;
      C.Refresh;

      R := Self.ClientRect;
      TabCtrl_AdjustRect(Self.Handle, False, @R);

      if TemaAtivo and (not FTransparentTab) then
      begin
        R2.Left := 0;
        R2.Top := 0;
        R2.Right := Self.ClientWidth;
        R2.Bottom := R.Top - 3;
        UxTheme.DrawThemeParentBackground(Self.Handle, C.Handle, @R2);
      end;

      for i := 0 to Count - 1 do
        if Pages[i] <> AP then
          Self.DesenharTab(C, i,  R.Top - 1 {AP.Top - 1});

      R.Left := 0;
      R.Top := R.Top - 1;
      R.Right := Self.ClientWidth;
      R.Bottom := R.Bottom + 1;

      C.Pen.Style := psSolid;
      C.Brush.Style := bsSolid;
      C.Brush.Color := EsquemaAtual.CorFundoPageControl;
      C.Pen.Color := EsquemaAtual.Cores[seNormal].CorBorda;
      C.Rectangle(R);

      Self.DesenharTab(C, Self.ActivePageIndex, AP.Top - 1);

//      Self.PaintControls(C.Handle, nil);
    end else
    begin
      R := Self.ClientRect;
      C.Pen.Style := psSolid;
      C.Brush.Style := bsSolid;
      C.Brush.Color := EsquemaAtual.CorFundoPageControl;
      C.Pen.Color := EsquemaAtual.Cores[seNormal].CorBorda;
      C.Rectangle(R);
    end;
  finally
    C.Handle := 0;
  end;
end;

procedure TRPageControl.DesenharTab(C: TCanvas; Indice, Altura: Integer);
var
  R, RTexto: TRect;
  Situacao: TSituacaoEsquema;
  Txt: String;
  Pagina: TTabSheet;
  RPagina: TRTabSheet;
  WPicture, HPicture, WTexto, HTexto, XPic, YPic, XTxt, YTxt, WConteudo: Integer;
  Pic: TPicture;
const
  ESPCAMENTO_IMAGEM_TEXTO = 8;
begin
  TabCtrl_GetItemRect(Self.Handle, Indice, R);

  if Pages[Indice].Visible  then
    Situacao := sePressionado
  else
  begin
    if ThemeControl(Self) and Self.IsMouseSobreControle(R) then
      Situacao := seHot
    else
      Situacao := seNormal;
  end;

//  R.Bottom := Altura;

  if Situacao = sePressionado then
  begin
    InflateRect(R, 2, 2);
//    R.Bottom := R.Bottom - 1;
//    OffsetRect(R, 0, 1);
    C.Brush.Color := EsquemaAtual.CorFundoTabSelecionado;
    C.Pen.Style := psSolid;
    C.Pen.Color := C.Brush.Color;
    C.Brush.Style := bsSolid;
    C.Rectangle(R);

//    InflateRect(R, 1, 1);
    C.Pen.Style := psSolid;
    C.Pen.Color := EsquemaAtual.Cores[seNormal].CorBorda;
    C.MoveTo(R.Right, R.Bottom - 1);
    C.LineTo(R.Right, R.Top);
    C.LineTo(R.Left, R.Top);
    C.LineTo(R.Left, R.Bottom);
  end else
  begin
    OffsetRect(R, 0, 1);
    R.Bottom := R.Bottom + 1;
    TRRPGDrawUtil.DrawBkgButton(C, R, Situacao, False, False, ebRetangular);

    InflateRect(R, -1, -1);
    OffsetRect(R, 1, 0);
  end;

  Pagina := Self.Pages[Indice];
  Txt := Pagina.Caption;
  WPicture  := 0;
  HPicture := 0;
  Pic := nil;

  if Pagina is TRTabSheet then
  begin
    RPagina := TRTabSheet(Pagina);

    if RPagina.PictureIcon.Graphic <> nil then
    begin
      Pic := RPagina.PictureIcon;
      WPicture := Pic.Width;
      HPicture := Pic.Height;
    end;
  end;

  WTexto := 0;
  HTexto := 0;

  if Txt <> '' then
  begin
    SelectObject(C.Handle, C.Font.Handle);
    RTexto := Rect(0, 0, 0, 0);
    C.TextRect(RTexto, Txt, [tfCalcRect, tfSingleLine]);
    WTexto := RTexto.Right - RTexto.Left;
    HTexto := RTexto.Bottom - RTexto.Top;
  end;

  WConteudo := WTexto + WPicture;

  if (WTexto > 0) and (WPicture > 0) then
    WConteudo := WConteudo + ESPCAMENTO_IMAGEM_TEXTO;

  XPic := R.Left + (R.Right - R.Left) div 2 - WConteudo div 2;
  YPic := R.Top + (R.Bottom - R.Top) div 2 - HPicture div 2;

  XTxt := XPic + WPicture;

  if (WTexto > 0) and (WPicture > 0) then
    XTxt := XTxt + ESPCAMENTO_IMAGEM_TEXTO;

  YTxt := R.Top + (R.Bottom - R.Top) div 2 - HTexto div 2;

  if Txt <> '' then
  begin
    RTexto := Rect(XTxt, YTxt, XTxt + WTexto, YTxt + HTexto);
    C.TextRect(RTexto, Txt, [tfCenter, tfVerticalCenter, tfSingleLine]);
  end;

  if Pic <> nil then
    C.Draw(XPic, YPic, Pic.Graphic);

  if Self.Focused and (Self.Pages[Indice].Visible) and (WConteudo > 0) then
  begin
    RTexto.Top := Min(YTxt, YPic);
    RTexto.Bottom := Max(YTxt + HTexto, YPic + HPicture);
    RTexto.Left := R.Left;
    RTexto.Right := R.Right;
    InflateRect(RTexto, -3, 1);
    TRRPGDrawUtil.DrawFocusRect(C, RTexto, seNormal);
  end;

  {if Txt <> '' then
  begin
    RTexto := R;

    SelectObject(C.Handle, C.Font.Handle);

    C.TextRect(RTexto, Txt, [tfCenter, tfVerticalCenter, tfSingleLine]);

    if Self.Focused and (Self.Pages[Indice].Visible) then
    begin
      C.TextRect(R, Txt, [tfCenter, tfCalcRect]);
      OffsetRect(RTexto, 0, (RTexto.Bottom - R.Bottom) div 2);
      RTexto.Bottom := RTexto.Top + (R.Bottom - R.Top);
      InflateRect(RTexto, -3, 1);
      TRRPGDrawUtil.DrawFocusRect(C, RTexto, seNormal);
    end;
  end;}
end;

destructor TRPageControl.Destroy;
begin
  FCanvasDraw.Free;
  inherited;
end;

function TRPageControl.IsMouseSobreControle(R: TRect): Boolean;
var
  PtMouse, PtOrigem, Pt: TPoint;
begin
  PtOrigem := Self.ClientOrigin;
  GetCursorPos(PtMouse);

  Pt.X := PtMouse.X - PtOrigem.X;
  Pt.Y := PtMouse.Y - PtOrigem.Y;

  Result := (Pt.X >= R.Left) and (Pt.Y >= R.Top) and (PT.X < R.Right) and
            (Pt.Y < R.Bottom);
end;

procedure TRPageControl.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Indice: Integer;
  Txt: String;
  Pagina: TTabSheet;
  Mouse: TPoint;
  Hit: TTCHitTestInfo;
begin
  inherited;

  if (csDesigning in Self.ComponentState) then
    Exit;

  Txt := '';

  Hit.pt.X := X;
  Hit.pt.Y := Y;

  Indice := TabCtrl_HitTest(Self.Handle, Hit);

  if (Hit.flags and TCHT_ONITEM = 0) or (Hit.flags = TCHT_NOWHERE) then
    Indice := -1;

  if (Indice >= 0) and (Indice <= Self.PageCount - 1) then
  begin
    Pagina := Self.Pages[Indice];

    if Pagina is TRTabSheet then
      Txt := TRTabSheet(Pagina).TabHint;
  end;

  if Self.Hint <> Txt then
  begin
    if Self.Hint <> '' then
      Application.CancelHint;

    Self.Hint := Txt;

    if Txt <> '' then
    begin
      GetCursorPos(Mouse);
      Application.ActivateHint(Mouse);
    end;
  end;
end;

procedure TRPageControl.NovaPagina(const Caption: String);
var
  nPagina: TRTabSheet;
  Radical: String;
  i: Integer;
begin
  nPagina := TRTabSheet.Create(Self.Owner);
  nPagina.PageControl := Self;

  i := 1;

  if Self.Owner = nil then
    Self.Name := 'tshPagina1'
  else
    while  Self.Owner.FindComponent('tshPagina' + IntToStr(i)) <> nil do
      Inc(i);

  Radical := 'Pagina' +  IntToStr(i);
  nPagina.Name := 'tsh' + Radical;

  if csDesigning in Self.ComponentState then
  begin
    if Caption = '' then
      nPagina.Caption := nPagina.Name;
  end;

  Self.ActivePage := nPagina;
end;

procedure TRPageControl.PaintWindow(DC: HDC);
begin
  Self.DesenharEm(DC);
end;

function TRPageControl.RemoverPagina: Boolean;
begin
  if Self.PageCount > 0 then
    Self.ActivePage.Free;

  Result := True;
end;

procedure TRPageControl.SetTransparentTab(const Value: Boolean);
begin
  if FTransparentTab <> Value then
  begin
    FTransparentTab := Value;

    if Self.HandleAllocated then
      Self.RecreateWnd();
  end;
end;

procedure TRPageControl.WMEraseBkg(var Message: TWMEraseBkgnd);
begin
  if FTransparentTab and not (csDesigning in Self.ComponentState) then
    Message.Result := 1
  else
    inherited;
end;

{ TRTabSheet }

{ TRTabSheet }

procedure TRTabSheet.ConfigurarVisual;
begin
  Self.Color := EsquemaAtual.CorFundoPageControl;
end;

constructor TRTabSheet.Create(AOwner: TComponent);
begin
  inherited;
  FCanvasDraw := TCanvas.Create;
  RegistrarNotScheme(Self, Self.ConfigurarVisual);
  FPictureIcon := TPicture.Create;
  FPictureIcon.OnChange := Self.PictureChanged;
end;

procedure TRTabSheet.DesenharEm(DC: HDC);
var
  C: TCanvas;
  R: TRect;
begin
  C := FCanvasDraw;
  C.Handle := DC;

  try
    C.Brush.Color := EsquemaAtual.CorFundoPageControl;
    C.Pen.Color := C.Brush.Color;
    C.Brush.Style := bsSolid;
    C.Pen.Style := psSolid;
    R := Self.ClientRect;

    C.Rectangle(R);

//    Self.PaintControls(DC, nil);
  finally
    C.Handle := 0;
  end;
end;

destructor TRTabSheet.Destroy;
begin
  FPictureIcon.Free;
  DesregistrarNotScheme(Self);
  FCanvasDraw.Free;
  inherited;
end;

procedure TRTabSheet.Loaded;
begin
  inherited;
  Self.ConfigurarVisual();
end;

procedure TRTabSheet.PaintWindow(DC: HDC);
begin
  Self.DesenharEm(DC);
end;

procedure TRTabSheet.PictureChanged(Sender: TObject);
begin
  if not (csLoading in Self.ComponentState) then
    if Self.PageControl <> nil then
      Self.PageControl.Invalidate;
end;

procedure TRTabSheet.SetPictureIcon(const Value: TPicture);
begin
  FPictureIcon.Assign(Value);
end;

procedure TRTabSheet.WMEraseBkg(var Message: TWMEraseBkgnd);
begin
  // Faz nada
  Self.DesenharEm(Message.DC);
  Message.Result := 1;
end;

procedure TRTabSheet.WMPrintClient(var Message: TWMPrintClient);
begin
  Self.DesenharEm(Message.DC);
  Message.Result := 1;
end;

end.
