unit RRPGPluginFicha;

interface

uses
  Windows, SysUtils, Classes, RRPGModulo, RRPGModuloDLL, RRPGBaseVisaoFicha,
  Controls, RRPGDataSet, DB, RRPGOwnerDatasets, Dialogs;
                                                            //System
type
  TReadOnlyEvent = procedure (const SomenteLeitura: Boolean) of object;

  TRRPGFicha = class(TDataModule, IRRPGFicha, IRRPGFichaVisao,
                     IDelphiRRPGOwnerDatasets)
  private
    FVisaoInstanciada: TFrmBaseVisaoFicha;
    FFichaVisaoController: IRRPGFichaVisaoController;

    // Eventos
    FBeforeCarregarFicha: TNotifyEvent;
    FBeforeSalvarFicha: TNotifyEvent;
    FBeforeLimparFicha: TNotifyEvent;
    FAfterCaregarFicha: TNotifyEvent;
    FAfterSalvarFicha: TNotifyEvent;
    FAfterLimparFicha: TNotifyEvent;
    FOnSetSomenteLeitura: TReadOnlyEvent;
    FRefCount: Integer;
    FFichaSomenteLeitura: Boolean;
  protected
    procedure LiberarInstanciaVisao; virtual;
    function GetVisaoClass: TFrmBaseVisaoFichaClass; virtual;
    function ObterInstanciaVisao(WindowHandle: Cardinal; const ForcarCriacao: Boolean = True): TFrmBaseVisaoFicha; virtual;

    // Inteface IRRPGFicha
    procedure CarregarDe(const Stream: IRRPGStream); virtual; stdcall;
    procedure SalvarEm(const Stream: IRRPGStream); virtual; stdcall;
    procedure SetReadOnly(const SomenteLeitura: Boolean); virtual; stdcall;
    procedure LimparFicha; virtual; stdcall;

    // Interface IRRPGFichaVisao
    procedure AcoplarVisaoEm(WindowHandle: Cardinal); virtual; stdcall;
    procedure DesacoplarVisao; virtual; stdcall;
    procedure SetFichaVisaoController(const FichaVisaoController: IRRPGFichaVisaoController); virtual; stdcall;
    procedure RedimensionarVisao(const Largura: Integer; const Altura: Integer); virtual; stdcall;

    // Interface
    procedure NotificarDataSetMudou(const DataSet: TObject);
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;

    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    function IRRPGFicha._AddRef = _AddRef;
    function IRRPGFichaVisao._AddRef = _AddRef;

    function IRRPGFicha._Release = _Release;
    function IRRPGFichaVisao._Release = _Release;

    class function NewInstance: TObject; override;

    property FichaVisaoController: IRRPGFichaVisaoController read FFichaVisaoController;
    property VisaoInstanciada: TFrmBaseVisaoFicha read FVisaoInstanciada;
  published
    property BeforeCarregarFicha: TNotifyEvent read FBeforeCarregarFicha write FBeforeCarregarFicha;
    property BeforeSalvarFicha: TNotifyEvent read FBeforeSalvarFicha write FBeforeSalvarFicha;
    property BeforeLimparFicha: TNotifyEvent read FBeforeLimparFicha write FBeforeLimparFicha;
    property AfterCaregarFicha: TNotifyEvent read FAfterCaregarFicha write FAfterCaregarFicha;
    property AfterSalvarFicha: TNotifyEvent read FAfterSalvarFicha write FAfterSalvarFicha;
    property AfterLimparFicha: TNotifyEvent read FAfterLimparFicha write FAfterLimparFicha;
    property OnSetSomenteLeitura: TReadOnlyEvent read FOnSetSomenteLeitura write FOnSetSomenteLeitura;
  end;

  TRRPGFichaClass = class of TRRPGFicha;

  TPluginFicha = class(TInterfacedObject, IRRPGPluginFicha)
  private
    FRRPGNomePlugin: AnsiString;
    FRRPGDescricaoPlugin: AnsiString;
    FRRPGIDPlugin: AnsiString;
    FFichaClass: TRRPGFichaClass;
  protected
    // Interface IRRPGPluginFicha
    function GetNome: PAnsiChar; virtual; stdcall;
    function GetDescricao: PAnsiChar; virtual; stdcall;
    function GetID: PAnsiChar; virtual; stdcall;
    procedure CriarFicha(out Ficha: IRRPGFicha); virtual; stdcall;

    procedure NotifyModuloAtivo(Ativar: Boolean); virtual;
  public
    constructor Create(const NomePlugin, DescricaoPlugin, IDPlugin: String;
                       const ClasseFicha: TRRPGFichaClass);
    destructor Destroy; override;                       
  public
    property RRPGNomePlugin: AnsiString read FRRPGNomePlugin write FRRPGNomePlugin;
    property RRPGDescricaoPlugin: AnsiString read FRRPGDescricaoPlugin write FRRPGDescricaoPlugin;
    property RRPGIDPlugin: AnsiString read FRRPGIDPlugin write FRRPGIDPlugin;
    property FichaClass: TRRPGFichaClass read FFichaClass write FFichaClass;
  end;

var
  PluginsFicha: TList; // de TPluginFicha

implementation

uses
  {$IF RTLVERSION >= 25}UITypes,{$IFEND}
  RRPGStreamUtils, RRPGDataSetUtil, Variants, Types;

{$R *.dfm}

procedure DestruirPluginsFicha;
var
  i: Integer;
begin
  for i := PluginsFicha.Count - 1 downto 0 do
    TPluginFicha(PluginsFicha[i]).Free;

  PluginsFicha.Clear;    
end;

{ TPluginFicha }

constructor TPluginFicha.Create(const NomePlugin, DescricaoPlugin,
  IDPlugin: String; const ClasseFicha: TRRPGFichaClass);
begin
  FRRPGNomePlugin := AnsiString(NomePlugin);
  FRRPGDescricaoPlugin := AnsiString(DescricaoPlugin);
  FRRPGIDPlugin := AnsiString(IDPlugin);
  FFichaClass := ClasseFicha;

  PluginsFicha.Add(Self);
  DelphiRRPGModulo.RegistrarMetodoNotify(Self.NotifyModuloAtivo);
end;

procedure TPluginFicha.CriarFicha(out Ficha: IRRPGFicha);
begin
  if FFichaClass <> nil then
    Ficha := FFichaClass.Create(nil)
  else
    Ficha := nil;  
end;

destructor TPluginFicha.Destroy;
begin
  DelphiRRPGModulo.DesregistrarMetodoNotify(Self.NotifyModuloAtivo);
  PluginsFicha.Remove(Self);
  inherited;
end;

function TPluginFicha.GetDescricao: PAnsiChar;
begin
  Result := PAnsiChar(FRRPGDescricaoPlugin);
end;

function TPluginFicha.GetID: PAnsiChar;
begin
  Result := PAnsiChar(FRRPGIDPlugin);
end;

function TPluginFicha.GetNome: PAnsiChar;
begin
  Result := PAnsiChar(FRRPGNomePlugin);
end;

procedure TPluginFicha.NotifyModuloAtivo(Ativar: Boolean);
var
  Central: IRRPGCentralPluginFichas;
begin
  if DelphiRRPGModulo.Central = nil then
    Exit;

  if DelphiRRPGModulo.Central.QueryInterface(IRRPGCentralPluginFichas, Central) = S_OK then
  begin
    if Ativar then
      Central.RegistrarPluginFicha(Self)
    else
      Central.DesregistrarPluginFicha(Self);
  end;
end;

{ TRRPGFicha }

procedure TRRPGFicha.AcoplarVisaoEm(WindowHandle: Cardinal);
var
  F: TFrmBaseVisaoFicha;
begin
  F := Self.ObterInstanciaVisao(WindowHandle);

  if F <> nil then
  begin
    F.Left := 0;
    F.Top := 0;
    F.Align := alClient;
    Windows.SetParent(F.Handle, WindowHandle);
    F.Visible := True;
  end;
end;

procedure TRRPGFicha.AfterConstruction;
begin
  inherited;
  InterlockedDecrement(FRefCount)
end;

procedure TRRPGFicha.CarregarDe(const Stream: IRRPGStream);
type
  TRFieldDef = packed record
    Nome: AnsiString;
    Tipo: TVarType;
    Field: TField;
  end;
var
  TempStr: AnsiString;
  Cp: TComponent;
  DS: TRRPGDataSet;
  AntigoSomenteLeitura: Boolean;
  LDataSets: TList;
  i: Integer;

  function BytesRestantes: Integer;
  begin
    Result := Stream.GetSize() - Stream.Position;
  end;

  procedure CarregarDataSet;
  var
    Defs: array of TRFieldDef;
    QtDefs, QtRegs, i, j, RecNoOriginal: Integer;
    V: Variant;
  begin
    Stream.Read(@QtDefs, SizeOf(QtDefs)); // Qt de Fields
    SetLength(Defs, QtDefs);

    for i := 0 to QtDefs - 1 do
    begin
      Defs[i].Nome := RStReadString(Stream);
      Stream.Read(@Defs[i].Tipo, SizeOf(Defs[i].Tipo));

      if DS <> nil then
        Defs[i].Field := DS.FindField(String(Defs[i].Nome))
      else
        Defs[i].Field := nil;
    end;

    Stream.Read(@QtRegs, SizeOf(QtRegs)); // Qt de Registros

    if DS <> nil then
      DS.First;

    for i := 0 to QtRegs - 1 do
    begin
      if DS <> nil then
        DS.Insert;

      for j := 0 to Length(Defs) - 1 do
      begin
        V := Null;
        RStReadVar(Stream, Defs[j].Tipo, V);

        if Defs[j].Field <> nil then
          try
            if Defs[j].Field.FieldKind = fkData then
              Defs[j].Field.AsVariant := V;
          except
            on E: Exception do
              MessageDlg('Carregando field ' + Defs[j].Field.FieldName +
                         ': ' + E.Message, mtError, [mbOK], 0);
          end;
      end;

      if DS <> nil then
        DS.Post;
    end;

    Stream.Read(@RecNoOriginal, SizeOf(RecNoOriginal));

    if DS <> nil then
      DS.RecNo := RecNoOriginal;
  end;
begin
  if Assigned(FBeforeCarregarFicha) then
    FBeforeCarregarFicha(Self);

  LDataSets := TList.Create;

  try
    try
      for i := 0 to Self.ComponentCount - 1 do
        if Self.Components[i] is TDataSet then
        begin
          TDataSet(Self.Components[i]).DisableControls;

          try
            LDataSets.Add(Self.Components[i]);
          except
            TDataSet(Self.Components[i]).EnableControls;
            raise;
          end;
        end;

      Self.LimparFicha();
      Stream.Position := 0;

      while BytesRestantes > 0 do
      begin
        TempStr := RStReadString(Stream);

        Cp := Self.FindComponent(String(TempStr));

        if Cp is TRRPGDataSet then
          DS := TRRPGDataSet(CP)
        else
          DS := nil;

        if DS <> nil then
          DS.DisableControls;

        try
          if DS <> nil then
          begin
            AntigoSomenteLeitura := DS.SomenteLeitura;
            DS.SomenteLeitura := False;
          end else
            AntigoSomenteLeitura := False;

          try
            CarregarDataSet();
          finally
            if DS <> nil then
              DS.SomenteLeitura := AntigoSomenteLeitura;
          end;
        finally
          if DS <> nil then
            DS.EnableControls;
        end;
      end;
    finally
      for i := LDataSets.Count - 1 downto 0 do
        TDataSet(LDataSets[i]).EnableControls;
    end;
  finally
    LDataSets.Free;
  end;

  if Assigned(FAfterCaregarFicha) then
    FAfterCaregarFicha(Self);
end;

procedure TRRPGFicha.DesacoplarVisao;
begin
  Self.LiberarInstanciaVisao();
end;

destructor TRRPGFicha.Destroy;
begin
  if FVisaoInstanciada <> nil then
    FreeAndNil(FVisaoInstanciada);

  inherited;
end;

function TRRPGFicha.GetVisaoClass: TFrmBaseVisaoFichaClass;
begin
  Result := nil;
end;

procedure TRRPGFicha.LiberarInstanciaVisao;
begin
  if FVisaoInstanciada <> nil then
    FreeAndNil(FVisaoInstanciada);
end;

procedure TRRPGFicha.LimparFicha;
var
  i: Integer;

  procedure LimparDataSet(DS: TRRPGDataSet);
  var
    AntigoSomenteLeitura: Boolean;
  begin
    if DS.Active then
    begin
      if DS.State in [dsEdit, dsInsert] then
        DS.Cancel;

      DS.DisableControls;

      try
        AntigoSomenteLeitura := DS.SomenteLeitura;
        DS.SomenteLeitura := False;

        try
          DS.LimparDataSet();
        finally
          DS.SomenteLeitura := AntigoSomenteLeitura;
        end;
      finally
        DS.EnableControls;
      end;
    end;
  end;
begin
  if Assigned(FBeforeLimparFicha) then
    FBeforeLimparFicha(Self);

  for i := 0 to Self.ComponentCount - 1 do
    if Self.Components[i] is TRRPGDataSet then
      LimparDataSet(TRRPGDataSet(Self.Components[i]));

  if Assigned(FAfterLimparFicha) then
    FAfterLimparFicha(Self);
end;

class function TRRPGFicha.NewInstance: TObject;
begin
  Result := inherited NewInstance;
  TRRPGFicha(Result).FRefCount := 1;
end;

procedure TRRPGFicha.NotificarDataSetMudou(const DataSet: TObject);
begin
  if FFichaVisaoController <> nil then
    FFichaVisaoController.NotificarAlteracao();
end;

{*******************************************************************************
*
*   FindGlobalComponent(const Name: String): TComponent;
*
*}

var
  _FindGCFNome: String;
  _FINDGCFObjeto: TComponent;

function FindGlobalComponentFicha(const Name: String): TComponent;
begin
  if UpperCase(Name) = _FindGCFNome then  // _FindGCFNome deve estar em maiusculo 
    Result := _FINDGCFObjeto
  else
    Result := nil;
end;

function TRRPGFicha.ObterInstanciaVisao(WindowHandle: Cardinal; const ForcarCriacao: Boolean = True): TFrmBaseVisaoFicha;
var
  Classe: TFrmBaseVisaoFichaClass;
  Temp: String;
begin
  if (FVisaoInstanciada = nil) and ForcarCriacao then
  begin
    Classe := Self.GetVisaoClass();

    if Classe <> nil then
    begin
      Temp := Self.ClassName;

      if Temp <> '' then
        if Temp[1] = 'T' then
          Temp := Copy(Temp, 2, Length(Temp));

      _FindGCFNome := UpperCase(Temp);
      _FINDGCFObjeto := Self;
      Classes.RegisterFindGlobalComponentProc(FindGlobalComponentFicha);

      try
{        if WindowHandle <> 0 then
          FVisaoInstanciada := Classe.CreateParented(SelWindowHandle)
        else}
        FVisaoInstanciada := Classe.Create(Self, WindowHandle);
      finally
        Classes.UnregisterFindGlobalComponentProc(FindGlobalComponentFicha);
      end;
    end;
  end;

  Result := FVisaoInstanciada;
end;

procedure TRRPGFicha.RedimensionarVisao(const Largura, Altura: Integer);
var
  F: TFrmBaseVisaoFicha;
begin
  F := Self.ObterInstanciaVisao(0, False);

  if F <> nil then
  begin
    F.Constraints.MaxWidth := Largura;
    F.Constraints.MaxHeight := Altura;
  end;
end;

procedure TRRPGFicha.SalvarEm(const Stream: IRRPGStream);
var
  i: Integer;

  procedure SalvarDataSet(DS: TRRPGDataSet);
  var
    Temp, i: Integer;
    TempWord: Word;
    F: TField;
    Bk: TBookmark;
    RecNoOriginal, CountRec, CountFields: Integer;
    PosCountRec, PosAtual: Int64;
  begin
    if not DS.Active then
      Exit;

    if DS.State in [dsEdit, dsInsert] then
      DS.UpdateRecord;

    DS.DisableControls;

    try
      if DS.State in [dsEdit, dsInsert] then
        DS.Post;

      Bk := DS.GetBookmark;

      try
        RStWriteString(Stream, AnsiString(DS.Name));
        DS.Last;  // Contar todos os RecordCount

        if DS.RecordCount = 0 then
        begin
          Temp := 0;
          Stream.Write(@Temp, SizeOf(Temp)); // Quantidade de Fields
          Stream.Write(@Temp, SizeOf(Temp)); // Quantidade de registros

          Temp := -1;
          Stream.Write(@Temp, SizeOf(Temp)); // Posio do cursor. Qual registro deve ser focado
        end else
        begin
          // Definio dos Campos
          CountFields := 0;

          for i := 0 to DS.FieldCount - 1 do
            if DS.Fields[i].FieldKind = fkData then
              Inc(CountFields);

          Temp := CountFields;
          Stream.Write(@Temp, SizeOf(Temp)); // Quantidade de Campos Fsicos

          for i := 0 to DS.FieldCount - 1 do
          begin
            F := DS.Fields[i];

            if F.FieldKind <> fkData then
              Continue;

            RStWriteString(Stream, AnsiString(F.FieldName)); // Nome do Field

            // Tipo do Field
            TempWord := RRPGDataSetUtil.FieldTypeToVariantType(F.DataType);
            Stream.Write(@TempWord, SizeOf(TempWord));
          end;

          // Quantidade de Registros
          Temp := 0;
          PosCountRec := Stream.Position;
          Stream.Write(@Temp, SizeOf(Temp));

          // Valores dos registros
          CountRec := 0;
          RecNoOriginal := Ds.RecNo;
          DS.First;

          while not DS.Eof do
          begin
            for i := 0 to DS.FieldCount - 1 do
              if DS.Fields[i].FieldKind = fkData then
                RStWriteField(Stream, DS.Fields[i]);

            Inc(CountRec);
            DS.Next;
          end;

          // Voltar atras, preenchendo a quantidade de registros
          PosAtual := Stream.Position;
          Stream.Position := PosCountRec;

          try
            Stream.Write(@CountRec, SizeOf(CountRec));
          finally
            Stream.Position := PosAtual;
          end;

          Stream.Write(@RecNoOriginal, SizeOf(RecNoOriginal)); // Posio do cursor. Qual registro deve ser focado
        end;
      finally
        if DS.BookmarkValid(Bk) then
        begin
          DS.GotoBookmark(Bk);
          DS.FreeBookmark(Bk);
        end;
      end;
    finally
      DS.EnableControls;
    end;
  end;
begin
  if Assigned(FBeforeSalvarFicha) then
    FBeforeSalvarFicha(Self);

  for i := 0 to Self.ComponentCount - 1 do
    if Self.Components[i] is TRRPGDataSet then
      SalvarDataSet(TRRPGDataSet(Self.Components[i]));

  if Assigned(FAfterSalvarFicha) then
    FAfterSalvarFicha(Self);
end;

procedure TRRPGFicha.SetFichaVisaoController(const FichaVisaoController: IRRPGFichaVisaoController);
begin
  FFichaVisaoController := FichaVisaoController;
end;

procedure TRRPGFicha.SetReadOnly(const SomenteLeitura: Boolean);
var
  i: Integer;
begin
  FFichaSomenteLeitura := SomenteLeitura;

  for i := 0 to Self.ComponentCount - 1 do
    if Self.Components[i] is TRRPGDataSet then
      TRRPGDataSet(Self.Components[i]).SomenteLeitura := SomenteLeitura;

  if Assigned(FOnSetSomenteLeitura) then
    FOnSetSomenteLeitura(SomenteLeitura);
end;

function TRRPGFicha._AddRef: Integer;
begin
  Result := InterlockedIncrement(FRefCount);
end;

function TRRPGFicha._Release: Integer;
begin
  Result := InterlockedDecrement(FRefCount);

  if FRefCount = 0 then
    Self.Free;
end;

initialization
  PluginsFicha := TList.Create;

finalization
  DestruirPluginsFicha();
  PluginsFicha.Free;

end.
