unit RRPGTrackBar;

interface

uses
  Windows, Messages, SysUtils, DB, Classes, Controls, Graphics,
  ComCtrls, Forms, URLMon, ActiveX, SyncObjs, StdCtrls, ExtCtrls, RRPGDrawUtil,
  CommCtrl;  // DBCtrls

type
  TRTrackBar = class(TTrackBar)
  private
    FCanvasDraw: TCanvas;
    FIsMousePressionadoThumb: Boolean;
  protected
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure DesenharEm(C: TCanvas); virtual;
    function IsMouseSobreControle(R: TRect): Boolean;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation

uses
  GraphUtil, UxTheme, RRPGUtil, Themes, RRPGColorScheme, GDIPlus;

{ TRTrackBar }

constructor TRTrackBar.Create(AOwner: TComponent);
begin
  inherited;
  FCanvasDraw := TCanvas.Create;
end;

procedure TRTrackBar.DesenharEm(C: TCanvas);
var
  R: TRect;
  Situacao: TSituacaoEsquema;
begin
  R := Self.ClientRect;

  // Fundo
  C.Brush.Style := bsSolid;
  C.Pen.Style := psSolid;
  C.Brush.Color := Self.Color;
  C.Pen.Color := Self.Color;
  C.Rectangle(R);

  // Canal

  SendMessage(Self.Handle, TBM_GETCHANNELRECT, 0, Integer(@R));
  C.Brush.Color := EsquemaAtual.TrackBar.CorChannel;
  C.Pen.Style := psClear;
  C.Rectangle(R);

  C.Pen.Style := psSolid;
  C.Pen.Color := EsquemaAtual.TrackBar.CorBordaChannel1;
  C.MoveTo(R.Left, R.Bottom);
  C.LineTo(R.Left, R.Top);
  C.LineTo(R.Right, R.Top);

  C.Pen.Color := EsquemaAtual.TrackBar.CorBordaChannel2;
  C.LineTo(R.Right, R.Bottom);
  C.LineTo(R.Left, R.Bottom);

  // Thumb

  SendMessage(Self.Handle, TBM_GETTHUMBRECT, 0, Integer(@R));

  if Self.Enabled then
    if FIsMousePressionadoThumb then
      Situacao := sePressionado
    else
      if Self.IsMouseSobreControle(R) then
        Situacao := seHot
      else
        Situacao := seNormal
  else
    Situacao := seDisabled;

  TRRPGDrawUtil.DrawBkgButton(C, R, Situacao, False, False, ebRetangular);

  if Self.Focused then
  begin
    R := Self.ClientRect;
    R.Bottom := R.Bottom - 1;
    R.Right := R.Right - 1;
    TRRPGDrawUtil.DrawFocusRect(C, R, seNormal);
  end;
end;

destructor TRTrackBar.Destroy;
begin
  FCanvasDraw.Free;
  inherited;
end;

function TRTrackBar.IsMouseSobreControle(R: TRect): Boolean;
var
  PtMouse, PtOrigem, Pt: TPoint;
begin
  PtOrigem := Self.ClientOrigin;
  GetCursorPos(PtMouse);

  Pt.X := PtMouse.X - PtOrigem.X;
  Pt.Y := PtMouse.Y - PtOrigem.Y;

  Result := (Pt.X >= R.Left) and (Pt.Y >= R.Top) and (PT.X < R.Right) and
            (Pt.Y < R.Bottom);
end;

procedure TRTrackBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var
  R: TRect;
begin
  inherited;

  if Button = mbLeft then
  begin
    SendMessage(Self.Handle, TBM_GETTHUMBRECT, 0, Integer(@R));

    if not FIsMousePressionadoThumb then
    begin
      FIsMousePressionadoThumb := (X >= R.Left) and (X < R.Right) and
                                  (Y >= R.Top) and (Y < R.Bottom);

      if FIsMousePressionadoThumb then
        Self.Invalidate;
    end;
  end;
end;

procedure TRTrackBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited;

  if Button = mbLeft then
  begin
    if FIsMousePressionadoThumb then
    begin
      FIsMousePressionadoThumb := False;
      Self.Invalidate;
    end;
  end;
end;

procedure TRTrackBar.WMPaint(var Message: TWMPaint);
var
  DC: HDC;
  PT: TPaintStruct;
begin
  DC := Message.DC;

  if DC = 0 then
    DC := BeginPaint(Self.Handle, PT);

  try
    FCanvasDraw.Handle := DC;

    try
      Self.DesenharEm(FCanvasDraw);
    finally
      FCanvasDraw.Handle := 0;
    end;
  finally
    if Message.DC = 0 then
      EndPaint(Self.Handle, PT);
  end;

  Message.Result := 1;
end;

end.
