unit RRPGUtil;

interface

uses
  Windows, SysUtils, Classes, Variants;

type
  TTempFileStream = class(TFileStream)
  private
    FArquivo: String;
  public
    property Arquivo: String read FArquivo;
  end;

  TRRPGUtil = class(TObject)
  public
    class procedure FreeTList(var Lista: TList);
    class procedure LimparTList(Lista: TList);
    class procedure InterpretarParametros(const S: String; Em: TStringList);
    class procedure ProcessarMsgs;
    class procedure ForcarJanelaApp(Handle: Cardinal; const Forcar: Boolean = True);
    class function LerDoArquivo(const Arq: String): String;
    class function UsuarioDoWindows: String;
    class function CriarTempFileStream: TTempFileStream;
    class function CriarTempFileName: String;
  end;

  TMemoryBuffer = class(TObject)
  private
    FMaxBytesInMemory: NativeInt;
    FBytesRemaining: NativeInt;
    FEspacoLixoNaFrente: NativeInt;
    FStream: TStream;
    FInMemory: Boolean;
    FPosition: NativeInt;

    procedure TruncLixo;
    procedure MigrarParaArquivo;
    procedure MigrarParaMemoria;
  public
    constructor Create(const MaxBytesInMemory: NativeInt);
    destructor Destroy; override;

    procedure Append(const Dados; const Tamanho: NativeInt);
    procedure AppendStream(Stream: TStream; const Tamanho: NativeInt);
    procedure AppendMemoryBuffer(const MB: TMemoryBuffer; const Tamanho: NativeInt);
    function Read(var Dados; const Tamanho: NativeInt): NativeInt;
    procedure Descartar(const QtBytesNoInicio: NativeInt);
    procedure Limpar;

    property BytesRemaining: NativeInt read FBytesRemaining;
    property Position: NativeInt read FPosition write FPosition;
  end;

function BoolToStrSN(const Valor: Boolean): String;
function IsNull(V: Variant): Boolean;
function AparentaSerEmail(const Email: String): Boolean;
function Coalesce(const Str1, Str2: String): String;


function WinPTBrStrCompare(const S1, S2: String): Boolean;
function WinPTBrGenCollateStr(const S: String; const CollateSymbols: Boolean = False): String;
function SubstituirEspacosPorUnderline(const S: String): String;

var
  IsMaiorIgualWindows7: Boolean = False;
  IsMaiorIgualWinVista: Boolean = False;

function TamanhoToStr(const Tam: Int64): String;

implementation

uses
  Math;

function WinPTBrStrCompare(const S1, S2: String): Boolean;
var
  L1, L2: NativeInt;
begin
  L1 := Length(S1);
  L2 := Length(S2);

  if L1 = L2 then
    if L1 = 0 then
      Result := True
    else
      Result := WinPTBrGenCollateStr(S1) = WinPTBrGenCollateStr(S2)
  else
    Result := False;
end;

function SubstituirEspacosPorUnderline(const S: String): String;
begin
  Result := StringReplace(S, ' ', '_',[rfReplaceAll]);
  Result := StringReplace(Result, #160, '_',[rfReplaceAll]);
end;

function WinPTBrGenCollateStr(const S: String; const CollateSymbols: Boolean = False): String;
var
  i: Integer;
  C: Char;
begin
  SetLength(Result, Length(S));

  if CollateSymbols then
  begin
    for i := 1 to Length(S) do
    begin
      C := S[i];

      case C of
        '', '', '', '', '', '', '', '', '', '', '', '': C := 'A';
        '', '': C := 'C';
        '', '', '', '', '', '', '', '': C := 'E';
        '', '', '', '', '', '', '', '': C := 'I';
        '', '': C := 'N';
        '', '', '', '', '', '', '', '', '', '': C := 'O';
        '', '', '', '', '', '', '', '': C := 'U';
        '', '', '': C := 'Y';
        #160: C := ' ';

        // Symbolos para fazer Collate
        '': C := 'X';

        else
          C := UpCase(C);
      end;

      Result[i] := C;
    end;
  end else
  begin
    for i := 1 to Length(S) do
    begin
      C := S[i];

      case C of
        '', '', '', '', '', '', '', '', '', '', '', '': C := 'A';
        '', '': C := 'C';
        '', '', '', '', '', '', '', '': C := 'E';
        '', '', '', '', '', '', '', '': C := 'I';
        '', '': C := 'N';
        '', '', '', '', '', '', '', '', '', '': C := 'O';
        '', '', '', '', '', '', '', '': C := 'U';
        '', '', '': C := 'Y';
        #160: C := ' ';

        else
          C := UpCase(C);
      end;

      Result[i] := C;
    end;
  end;
end;

function Coalesce(const Str1, Str2: String): String;
begin
  if Trim(Str1) <> '' then
    Result := Str1
  else
    Result := Str2;
end;

function AparentaSerEmail(const Email: String): Boolean;
var
  Temp, Temp2: String;
  PArroba, PPonto: Integer;
begin
  Temp := Trim(Email);

  if Temp <> '' then
  begin
    PArroba := Pos('@', Temp);

    if (PArroba > 1) and (PArroba < Length(Email)) then
    begin
      Temp2 := Copy(Temp, PArroba + 1, Length(Email));
      PPonto := Pos('.', Temp2);
      Result := (PPonto > 1) and (PPonto < Length(Temp2));

      if Result then
        Result := (Pos(';', Temp) = 0) and (Pos(',', Temp) = 0);
    end else
      Result := False;
  end else
    Result := False;
end;

function TamanhoToStr(const Tam: Int64): String;
begin
  if Tam < 1024 then
    Result := Format('%d bytes', [Tam])
  else
    if Tam < 1024 * 1024  then
      Result := Format('%2.2f kb', [Tam / 1024])
  else
    if Tam < 1024 * 1024 * 1024  then
      Result := Format('%2.2f mb', [Tam / (1024 * 1024)])
  else
    Result := Format('%2.2f gb', [Tam / (1024 * 1024 * 1024)]);
end;

procedure InicializarVersaoWindows;
var
  V: TOSVersionInfoEx;
begin
  FillChar(V, SizeOf(V), 0);
  V.dwOSVersionInfoSize := SizeOf(V);
  Windows.GetVersionEx(V);

  if V.dwMajorVersion >= 6 then
  begin
    IsMaiorIgualWinVista := True;

    if V.dwMajorVersion = 6 then
    begin
      if V.dwMinorVersion >= 1 then
        IsMaiorIgualWindows7 := True;
    end else
      IsMaiorIgualWindows7 := True;
  end;
end;

function BoolToStrSN(const Valor: Boolean): String;
begin
  if Valor then
    Result := 'S'
  else
    Result := 'N';
end;

function IsNull(V: Variant): Boolean;
begin
  Result := (V = Null) or (VarIsEmpty(V));
end;

{ TRRPGUtil }

class procedure TRRPGUtil.FreeTList(var Lista: TList);
begin
  if Lista = nil then
    Exit;

  try
    TRRPGUtil.LimparTList(Lista);
  finally
    Lista.Free;
    Lista := nil;
  end;
end;

class procedure TRRPGUtil.InterpretarParametros(const S: String;
  Em: TStringList);
var
  i: Integer;
  C: Char;
  Temp: String;
  InString, AntAspas: Boolean;

  procedure AddTemp;
  begin
    if Temp = '' then
      Exit;

    Em.Add(Temp);
    Temp := '';
  end;
begin
  Temp := '';
  InString := False;
  AntAspas := False;

  for i := 1 to Length(S) do
  begin
    C := S[i];

    if AntAspas then
    begin
      if C = '"' then
      begin
        if InString then
          Temp := Temp + '"';

        Continue;
      end else
        InString := not InString;

      AntAspas := False;
    end;

    case C of
      ';':
        begin
          if InString then
            Temp := Temp + C
          else
            AddTemp();
        end;

      '"': AntAspas := True;

      else
        Temp := Temp + C;
    end;
  end;

  AddTemp();
end;

var
  GenUniqueTempFile: Integer;

class function TRRPGUtil.CriarTempFileName: String;
var
  TempPath, TempFile: String;
  U: Integer;
begin
  SetLength(TempPath, 1024);
  SetLength(TempPath, GetTempPath(Length(TempPath), PChar(TempPath)));

  U := InterlockedIncrement(GenUniqueTempFile);

  SetLength(TempFile, 1024);
  GetTempFileName(PChar(TempPath), 'SC2', U, PChar(TempFile));
  Result := PChar(TempFile);
end;

class function TRRPGUtil.CriarTempFileStream: TTempFileStream;
var
  TempFile: String;
  H: Cardinal;
begin
  TempFile := CriarTempFileName();

  H := CreateFile(PChar(TempFile),
                  GENERIC_READ or GENERIC_WRITE,
                  0,
                  nil,
                  CREATE_ALWAYS,
                  FILE_ATTRIBUTE_TEMPORARY or FILE_FLAG_DELETE_ON_CLOSE or
                  FILE_FLAG_SEQUENTIAL_SCAN, 0);

  if H = INVALID_HANDLE_VALUE then
    RaiseLastOSError;

  Result := TTempFileStream.Create(H);
  Result.FArquivo := TempFile;
end;

class procedure TRRPGUtil.ForcarJanelaApp(Handle: Cardinal; const Forcar: Boolean = True);
var
  Estilo: Cardinal;
begin
  Estilo := GetWindowLong(Handle, GWL_EXSTYLE);

  if Forcar then
    Estilo := Estilo or WS_EX_APPWINDOW
  else
    Estilo := Estilo and (not WS_EX_APPWINDOW);

  SetWindowLong(Handle, GWL_EXSTYLE, Estilo);
end;

class procedure TRRPGUtil.ProcessarMsgs;
var
  Msg: tagMSG;
begin
  while Windows.PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do
  begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
end;

class function TRRPGUtil.LerDoArquivo(const Arq: String): String;
var
  F: TFileStream;
begin
  if (Arq = '') or (not FileExists(Arq)) then
  begin
    Result := '';
    Exit;
  end;

  F := TFileStream.Create(Arq, fmOpenRead or fmShareDenyNone);

  try
    F.Position := 0;

    SetLength(Result, F.Size);
    F.Read(Result[1], F.Size);
  finally
    F.Free;
  end;
end;

class procedure TRRPGUtil.LimparTList(Lista: TList);
var
  i: Integer;
begin
  for i := 0 to Lista.Count - 1 do
    TObject(Lista[i]).Free;

  Lista.Clear;
end;

class function TRRPGUtil.UsuarioDoWindows: String;
var
  Tam: Cardinal;
begin
  SetLength(Result, 512);
  Tam := Length(Result);

  if Windows.GetUserName(PChar(Result), Tam) then
    SetLength(Result, Tam)
  else
    Result := '';

  Result := Trim(Result);
end;

{ TBufferMemory }

procedure TMemoryBuffer.Append(const Dados; const Tamanho: NativeInt);
begin
  if FInMemory then
    if FBytesRemaining + Tamanho + FEspacoLixoNaFrente > FMaxBytesInMemory then
    begin
      // Vai estourar o tamanho mximo na memria.. Vamos ver o que podemos fazer..

      if (FEspacoLixoNaFrente > 0) and (FBytesRemaining + Tamanho <= FMaxBytesInMemory) then
        Self.TruncLixo() // Fazer Trunc do Espao lixo resolver o problema
      else
        Self.MigrarParaArquivo();  // Vamos ter que migrar para um arquivo
    end;

  FStream.Position := FEspacoLixoNaFrente + FBytesRemaining;
  FStream.Write(Dados, Tamanho);
  FBytesRemaining := FBytesRemaining + Tamanho;
end;

procedure TMemoryBuffer.AppendMemoryBuffer(const MB: TMemoryBuffer;
  const Tamanho: NativeInt);
begin
  MB.FStream.Position := MB.FEspacoLixoNaFrente + MB.FPosition;
  Self.AppendStream(MB.FStream, Tamanho);
end;

procedure TMemoryBuffer.AppendStream(Stream: TStream; const Tamanho: NativeInt);
begin
  if FInMemory then
    if FBytesRemaining + Tamanho + FEspacoLixoNaFrente > FMaxBytesInMemory then
    begin
      // Vai estourar o tamanho mximo na memria.. Vamos ver o que podemos fazer..

      if (FEspacoLixoNaFrente > 0) and (FBytesRemaining + Tamanho <= FMaxBytesInMemory) then
        Self.TruncLixo() // Fazer Trunc do Espao lixo resolver o problema
      else
        Self.MigrarParaArquivo();  // Vamos ter que migrar para um arquivo
    end;

  FStream.Position := FEspacoLixoNaFrente + FBytesRemaining;
  FStream.CopyFrom(Stream, Tamanho);
  FBytesRemaining := FBytesRemaining + Tamanho;
end;

constructor TMemoryBuffer.Create(const MaxBytesInMemory: NativeInt);
begin
  FMaxBytesInMemory := MaxBytesInMemory;
  FBytesRemaining := 0;
  FEspacoLixoNaFrente := 0;
  FInMemory := True;
  FStream := TMemoryStream.Create;
end;

procedure TMemoryBuffer.Descartar(const QtBytesNoInicio: NativeInt);
var
  QtDescartar, MaxMemDiv2: NativeInt;
begin
  QtDescartar := QtBytesNoInicio;

  if QtDescartar > FBytesRemaining then
    QtDescartar := FBytesRemaining;

  FEspacoLixoNaFrente := FEspacoLixoNaFrente + QtDescartar;
  FBytesRemaining := FBytesRemaining - QtDescartar;

  MaxMemDiv2 := FMaxBytesInMemory div 2;

  if FBytesRemaining = 0 then
    FEspacoLixoNaFrente := 0;

  FPosition := FPosition - QtDescartar;

  if FPosition < 0 then
    FPosition := 0;

  if FInMemory then
  begin
    if FEspacoLixoNaFrente >= MaxMemDiv2 then
      Self.TruncLixo();
  end else
  begin
    if FBytesRemaining < MaxMemDiv2 then
      Self.MigrarParaMemoria()
    else
      if FEspacoLixoNaFrente >= Max(512 * 1024, Round(FBytesRemaining * 0.25)) then
        Self.TruncLixo();
  end;
end;

destructor TMemoryBuffer.Destroy;
begin
  FStream.Free;
  inherited;
end;

procedure TMemoryBuffer.Limpar;
begin
  Self.Descartar(Self.BytesRemaining);
end;

procedure TMemoryBuffer.MigrarParaArquivo;
var
  NS: TStream;
begin
  NS := TRRPGUtil.CriarTempFileStream();
  NS.Position := 0;

  FStream.Position := FEspacoLixoNaFrente;
  NS.CopyFrom(FStream, FBytesRemaining);

  FStream.Free;
  FStream := NS;
  FEspacoLixoNaFrente := 0;

  FInMemory := False;
end;

procedure TMemoryBuffer.MigrarParaMemoria;
var
  NS: TStream;
begin
  NS := TMemoryStream.Create;

  FStream.Position := FEspacoLixoNaFrente;
  NS.CopyFrom(FStream, FBytesRemaining);

  FStream.Free;
  FStream := NS;
  FEspacoLixoNaFrente := 0;

  FInMemory := True;
end;

function TMemoryBuffer.Read(var Dados; const Tamanho: NativeInt): NativeInt;
var
  QtALer: NativeInt;
begin
  QtALer := Tamanho;

  if QtALer + FPosition > FBytesRemaining then
    QtALer := FBytesRemaining - FPosition;

  Result := QtALer;
  FStream.Position := FEspacoLixoNaFrente + FPosition;
  FStream.Read(Dados, Tamanho);
  FPosition := FPosition + QtALer;
end;

procedure TMemoryBuffer.TruncLixo;
var
  Buffer: Pointer;
  i, QtBlocos, Restante, SrcIndex, DestIndex: NativeInt;
const
  TAM_BUFFER_TRUNC = 8 * 1024;
begin
  Buffer := nil;

  try
    QtBlocos := FBytesRemaining div TAM_BUFFER_TRUNC;
    Restante := FBytesRemaining mod TAM_BUFFER_TRUNC;

    SrcIndex := FEspacoLixoNaFrente;
    DestIndex := 0;

    for i := 1 to QtBlocos do
    begin
      if Buffer = nil then
        GetMem(Buffer, TAM_BUFFER_TRUNC);

      FStream.Position := SrcIndex;
      FStream.Read(Buffer^, TAM_BUFFER_TRUNC);

      FStream.Position := DestIndex;
      FStream.Write(Buffer^, TAM_BUFFER_TRUNC);

      SrcIndex := SrcIndex + TAM_BUFFER_TRUNC;
      DestIndex := DestIndex + TAM_BUFFER_TRUNC;
    end;

    if Restante > 0 then
    begin
      if Buffer = nil then
        GetMem(Buffer, Restante);

      FStream.Position := SrcIndex;
      FStream.Read(Buffer^, TAM_BUFFER_TRUNC);

      FStream.Position := DestIndex;
      FStream.Write(Buffer^, TAM_BUFFER_TRUNC);
    end;

    FStream.Position := 0;
    FStream.Size := FBytesRemaining;
    FEspacoLixoNaFrente := 0;
  finally
    if Buffer <> nil then
      FreeMem(Buffer);
  end;
end;

initialization
  InicializarVersaoWindows();

end.
